/**
 * 
 */
package roneditor.gef.actions;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.clipboard.ClipboardUtil;
import roneditor.gef.editparts.ron.RONEditPart;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONArc;
import ronmodel.Token;

/**
 * This Action any kind model in form of a serialized String into the systems clipboard
 * @author choeger
 *
 */
public class CopyModelToClipboardAction extends SelectionAction {
	static public final String ID = "Copy ON to Clipboard";

	private Collection source;
	private int copyType = 0;
	
	/**
	 * 
	 */
	public CopyModelToClipboardAction(IWorkbenchPart part ) {
		super(part);
		setId(ActionFactory.COPY.getId());
		super.setText("Copy");
		super.setDescription("Copy ObjectNet to System clipboard");
		super.setToolTipText("Copies the selected ObjectNet to the clipboard");
	}
	
	/**
	 * just serialize the source...
	 * and use swt to clipboard it!
	 */
	public void run()
	{
		Collection copyCollection = EcoreUtil.copyAll(source);
		String clipString = ClipboardUtil.copyElementsToString(copyCollection, null, new NullProgressMonitor());		
		Object[] data = {clipString};
		TextTransfer transferType = TextTransfer.getInstance();
		Transfer[] types = {transferType};
		Clipboard clipboard = new Clipboard(Display.getCurrent());
		clipboard.setContents(data,types);
		PasteModelAction.lastCopied = copyType;
	}
	
	@SuppressWarnings("unchecked")
	public boolean calculateEnabled()
	{
		final StructuredSelection selection = ((StructuredSelection) getSelection());

		
		if (selection == null || selection.size() == 0)
			return false;
		
		source = new HashSet();
		Object[] array = selection.toArray();
		for (int i = 0; i < array.length; i++)
			if (array[i] instanceof AbstractEditPart)	
			{
				// do not copy the complete editor
				if (array[i] instanceof RONEditPart) return false;

				// default action is to copy objectnets
				copyType = PasteModelAction.COPY_OBJECT_NET;
				
				// if an arc shall be copied, we ensure that source and target will also be copied (closure condition)
				Object model = ((AbstractEditPart)array[i]).getModel();
				if (model instanceof ONArc) 
				{
					ONArc arc = (ONArc)model;
					source.add(arc.getSource());
					source.add(arc.getTarget());
				}
				
				/*
				 * HLPlaces
				 */
				if (model instanceof HLPlace)
					copyType = PasteModelAction.COPY_HL_MODEL;
				
				/*
				 * HLArcs
				 */
				if (model instanceof HLArc)
				{
					HLArc arc = (HLArc)model;
					source.add(arc.getSource());
					source.add(arc.getTarget());
					copyType = PasteModelAction.COPY_HL_MODEL;			
				}
				
				/*
				 * HLTransitions
				 */
				if (model instanceof HLTransition)
					copyType = PasteModelAction.COPY_HL_MODEL;				

				/*
				 * Tokens
				 */
				if (model instanceof Token)
				{
					Token token = (Token)model;
					HLPlace place = token.getPlace();
					if (place != null && place.getType() == HLPlaceType.get(HLPlaceType.NET)) 
						copyType = PasteModelAction.COPY_TOKEN_NET;
					else
						copyType = PasteModelAction.COPY_TOKEN_RULE;

				}
				
				source.add(model);		
			}	
	
		return source.size() != 0;
	}
	
}
