 package roneditor.gef.actions;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.ron.CreateTokenAction;
import roneditor.gef.commands.objectnet.PasteObjectNetCommand;
import roneditor.gef.commands.ron.CreateTokenCommand;
import roneditor.gef.commands.ron.PasteHLModelCommand;
import roneditor.gef.commands.ron.PasteTokenCommand;
import roneditor.gef.editparts.objectnet.ObjectNetEditPart;
import roneditor.gef.editparts.ron.HLPlaceEditPart;
import roneditor.gef.editparts.ron.RONEditPart;
import roneditor.model.RONModelManager;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;

/**
 * This action pastes the clipboard into the current edit part if allowed
 * @author choeger
 *
 */
public class PasteModelAction extends SelectionAction {
	public static String ID = "Paste ObjectNet";
	public static int lastCopied;
	protected static int COPY_OBJECT_NET = 1;
	protected static int COPY_TOKEN_RULE = 2;
	protected static int COPY_TOKEN_NET = 3;	
	protected static int COPY_HL_MODEL = 4;
	
	
	private String CREATE_TOKEN_ID = CreateTokenAction.ID;
	
	private ObjectNetEditPart target;
	private HLPlaceEditPart placeTarget;
	private RONEditPart ronTarget;
	private String textData = null;
	
	public PasteModelAction(IWorkbenchPart part) {
		// TODO Auto-generated constructor stub
		super(part);
		setId(ActionFactory.PASTE.getId());
		super.setText("Paste");
		super.setDescription("Paste ObjectNet from System clipboard");
		super.setToolTipText("Pastes the selected ObjectNet to the clipboard");
	}
	
	public void run()
	{    
    	if (textData == null) return;
	    if (target != null)
	    {
	    	execute(new PasteObjectNetCommand(textData,(ObjectNet)target.getModel()));
	    	return;
	    }
	    
	    if (placeTarget != null)
	    {	
		    /*
		     * create new Token with the clipboard content
		     */
	    	if (lastCopied == COPY_OBJECT_NET)
	    	{
	    		CompoundCommand command = new CompoundCommand();

	    		ObjectNet net =  RONModelManager.getFactory().createObjectNet();
	    		CreateTokenCommand createTokenCommand = new CreateTokenCommand(placeTarget.getCastedModel(),net);
	    		createTokenCommand.setName("pasted");

	    		command.add(createTokenCommand);
	    		command.add(new PasteObjectNetCommand(textData,net));
	    		execute(command);
	    		return;
	    	}

	    	if ((lastCopied == COPY_TOKEN_NET) || (lastCopied == COPY_TOKEN_RULE))
	    	{
	    		execute(new PasteTokenCommand(textData,placeTarget.getCastedModel()));
		    	return;
		    }
	    	
	    	return;
	    }
	    
	    if (ronTarget != null)
	    {
	    	if (lastCopied == COPY_HL_MODEL)
	    	{
	    		execute(new PasteHLModelCommand(textData,ronTarget.getCastedModel()));
	    		return;	    		
	    	}
	    	return;
	    }
	    
	}
	
	public boolean calculateEnabled()
	{
	    Clipboard clipboard = new Clipboard(Display.getCurrent());
	    TextTransfer textTransfer = TextTransfer.getInstance();
	    textData = (String)clipboard.getContents(textTransfer);
  
	    if (textData == null) return false;
	    
	    target = null;
		final StructuredSelection selection = ((StructuredSelection) getSelection());
		if (selection == null)
			return false;

		if (selection.size() != 1) {
			return false;
		}		
		final Object selectionElement = selection.getFirstElement();
	
	
		if (selectionElement instanceof ObjectNetEditPart)
		{
			if (lastCopied == COPY_OBJECT_NET)
			{
				target = (ObjectNetEditPart)selectionElement;			
				return true;
			}
			else return false;
		}	
		
		if (selectionElement instanceof HLPlaceEditPart)
		{
			HLPlaceEditPart part = (HLPlaceEditPart)selectionElement;
			if (lastCopied == COPY_OBJECT_NET)
			{
				if (part.getCastedModel().getType() != HLPlaceType.get(HLPlaceType.NET)) return false; 
				
				target = null;
				placeTarget = part;
				ronTarget = null;
				return true;
			}
			
			if (lastCopied == COPY_TOKEN_NET)
			{
				if (part.getCastedModel().getType() != HLPlaceType.get(HLPlaceType.NET)) return false; 
				target = null;
				placeTarget = part;
				ronTarget = null;
				return true;
			}
			
			if (lastCopied == COPY_TOKEN_RULE)
			{
				if (part.getCastedModel().getType() != HLPlaceType.get(HLPlaceType.RULE)) return false; 
				target = null;
				placeTarget = part;
				ronTarget = null;
				return true;
			}			
			return false;
		}	
		
		if (selectionElement instanceof RONEditPart)
		{
			RONEditPart part = (RONEditPart)selectionElement;
			if (lastCopied == COPY_OBJECT_NET)
			{
				//TODO: sollen ONs mit neuen HLPlaces + neuen Tokens gepasted werden können?
				return false;
			}
			
			if (lastCopied == COPY_TOKEN_NET)
			{
				//TODO: neuen HLPlace erstellen und Token einfügen (?)
				return false;
			}

			if (lastCopied == COPY_TOKEN_RULE)
			{
				//TODO: neuen HLPlace erstellen und Token einfügen (?)
				return false;
			}

			if (lastCopied == COPY_HL_MODEL)
			{
				placeTarget = null;
				target = null;
				ronTarget = part;
				return true;
			}
			return false;
		}
		return false;
		
	}
	

}
