package roneditor.gef.actions.objectnet;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import roneditor.gef.editpolicies.objectnet.ONTransitionEditPolicy;

/**
 * Action for firing a PetriNet's Transition.
 */
public class FireTransitionAction extends SelectionAction {

	public static final String ID = "roneditor.actions.FireTransitionAction";

	public static final String LABEL = "Fire Transition";

	public static final String DESCRIPTION = "Fire a transition by redistributing tokens";

	public FireTransitionAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(LABEL);
		setDescription(DESCRIPTION);
		setToolTipText(DESCRIPTION);
	}

	/**
	 * Gets a Command and executes it.
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		execute(getFireCommand());
	}

	/**
	 * Gets the command first the selected TransitionEditPart which delivers a
	 * Command for the "Fire" request since {@link #calculateEnabled()} must
	 * have returned true.
	 */
	private Command getFireCommand() {
		final StructuredSelection selection = ((StructuredSelection) getSelection());
		final ONTransitionEditPart part = (ONTransitionEditPart) selection.getFirstElement();
		return part.getCommand(new Request(ONTransitionEditPolicy.FIRE_REQUEST));
	}

	/**
	 * Check if one and only one {@link ONTransitionEditPart} is selected and
	 * verify it can fire.
	 * 
	 * @return true if we have selected a transition an if it can fire, false
	 *         otherwise.
	 * 
	 * @see ONTransitionEditPart#canFire()
	 */
	@Override
	protected boolean calculateEnabled() {
		final StructuredSelection selection = ((StructuredSelection) getSelection());

		if (selection == null || selection.size() != 1)
			return false;

		final Object selectionElement = selection.getFirstElement();
		if (!(selectionElement instanceof ONTransitionEditPart)) {
			return false;
		}

		final ONTransitionEditPart transitionPart = (ONTransitionEditPart) selectionElement;
		return transitionPart.canFire();
	}

}
