package roneditor.gef.actions.objectnet;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.objectnet.ObjectNetEditPartGraphLayoutCommand;
import roneditor.gef.editparts.objectnet.ObjectNetEditPart;
import roneditor.gef.editpolicies.objectnet.ObjectNetEditPartGraphLayoutEditPolicy;

/**
 * The <code>ObjectNetEditPartGraphLayoutAction</code> is an
 * {@link SelectionAction} for an {@link ObjectNetEditPart}. When a single
 * {@link ObjectNetEditPart} is selected, the
 * {@link ObjectNetEditPartGraphLayoutCommand} will be generated by the
 * {@link ObjectNetEditPartGraphLayoutEditPolicy}.
 * 
 * @author j
 */
public class ObjectNetEditPartGraphLayoutAction extends SelectionAction {

	public static final String ID = "roneditor.actions.ObjectNetEditPartGraphLayoutAction";

	public static final String LABEL = "Redistribute nodes";

	public static final String DESCRIPTION = "Redistribute the nodes of an Object Network with a directed graph layout";

	/**
	 * Creates a <code>ObjectNetEditPartGraphLayoutAction</code> and
	 * associates it with the given workbench part.
	 * 
	 * @param part
	 *            the workbench part
	 */
	public ObjectNetEditPartGraphLayoutAction(IWorkbenchPart part) {
		super(part);
	}

	/**
	 * @see org.eclipse.gef.ui.actions.EditorPartAction#init()
	 */
	@Override
	protected void init() {
		setId(ID);
		setText(LABEL);
		setDescription(DESCRIPTION);
		setToolTipText(DESCRIPTION);
	}

	/**
	 * Gets the Graph Layout Command and executes it.
	 */
	@Override
	public void run() {
		execute(getGraphLayoutCommand());
	}

	/**
	 * Return the {@link ObjectNetEditPartGraphLayoutCommand}.
	 */
	private Command getGraphLayoutCommand() {
		final StructuredSelection selection = ((StructuredSelection) getSelection());
		final ObjectNetEditPart part = (ObjectNetEditPart) selection.getFirstElement();
		return part.getCommand(new Request(ObjectNetEditPartGraphLayoutEditPolicy.REQUEST));
	}

	/**
	 * Check if a single {@link ObjectNetEditPart} is selected.
	 */
	@Override
	protected boolean calculateEnabled() {
		final StructuredSelection selection = ((StructuredSelection) getSelection());
		if (selection == null)
			return false;

		if (selection.size() != 1) {
			return false;
		}

		final Object selectionElement = selection.getFirstElement();
		if (!(selectionElement instanceof ObjectNetEditPart)) {
			return false;
		}

		return true;
	}

}
