package roneditor.gef.actions.ron;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.ron.CopyTokenToHLPlaceCommand;
import roneditor.gef.commands.ron.RemoveTokenFromHLPlaceCommand;
import roneditor.gef.commands.rule.ApplyRuleCommand;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.model.HLTransitionUtil;
import roneditor.model.ObjectNetUtil;
import roneditor.model.RONModelManager;
import roneditor.ui.dialog.AbstractMatchDialog;
import roneditor.ui.dialog.MatchDialog;
import ronmodel.Codomain;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * This is a sample action to demonstrate AbstractMatchDialog. It is enabled if
 * the RON in the background of the tree viewer is selected. It just grabs all
 * rules and object nets in the RON and puts the result of the rule application
 * on the HLPlace that hosts the selected object net.
 * 
 * @author Tony Modica
 */
public class ApplyRuleDialogAction extends SelectionAction {

	public static final String ID = "roneditor.dialogs.match.ApplyRuleDialogAction";

	public static final String NAME = "Match Dialog";

	public ApplyRuleDialogAction(final IWorkbenchPart part) {
		super(part);
		System.out.println("ApplyRuleAction.ApplyRuleAction() " + ID);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) (getSelectedObjects().get(0));
		HLTransition transition = transitionEditpart.getCastedModel();

		
		List<ObjectNet> tokens = HLTransitionUtil.collectSourceNetTokens(transition);
		ObjectNet unionObjectNet = RONModelManager.getFactory().createObjectNet();
		ObjectNetUtil.unionIntoObjectNet(tokens, unionObjectNet);

		// apply rule to unionObjectNet
		List<HLPlace> incomingRulePlaces = HLTransitionUtil.getIncomingRulePlaces(transition);
		Rule ruleToApply = (Rule) incomingRulePlaces.get(0).getTokens().get(0);
		
		// get all rules and object nets in the RON
		final ArrayList<Rule> rules = new ArrayList<Rule>();
		final ArrayList<ObjectNet> objectNets = new ArrayList<ObjectNet>();

		rules.add(ruleToApply);
		objectNets.add(unionObjectNet);
		
		// open dialog
		final AbstractMatchDialog dialog = new MatchDialog(rules, objectNets);
		dialog.open();
		if (dialog.isCanceled()) {
			return;
		}

		// fetch information from the dialog
		final Rule rule = dialog.getCurrentRule();
		final Codomain objectNet = dialog.getObjectNetCopy();
		final HLPlace parentPlace = dialog.getHLPlace();
		final HashMap<ONNode, ONNode> match = dialog.getMatchToApply();

		CompoundCommand compoundCommand = new CompoundCommand();
		
		// apply the selected rule to the selected object net
		ApplyRuleCommand applyRuleCommand = new ApplyRuleCommand(objectNet, rule, match);
		compoundCommand.add(applyRuleCommand);

		// Copy rule to the new outgoing tokens on HLPlace.RULE
		List<HLPlace> outgoingRulePlaces = HLTransitionUtil.getOutgoingRulePlaces(transition);
		for (HLPlace place : outgoingRulePlaces) {
			compoundCommand.add(new CopyTokenToHLPlaceCommand(rule, place));
		}

		// remove rule from the HLPlace of Type RULE
		compoundCommand.add(new RemoveTokenFromHLPlaceCommand(rule));

		// Copy unionObjectNet to the new outgoing tokens on HLPlace.NET
		List<HLPlace> outgoingNetPlaces = HLTransitionUtil.getOutgoingNetPlaces(transition);
		for (HLPlace place : outgoingNetPlaces) {
			compoundCommand.add(new CopyTokenToHLPlaceCommand((Token) unionObjectNet, place));
		}

		// remove tokens/ObjectNets from place
		for (ObjectNet t : tokens) {
			compoundCommand.add(new RemoveTokenFromHLPlaceCommand(t));
		}
		getCommandStack().execute(compoundCommand);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link HLTransition} of kind <i>APPLYRULE</i>.
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelection() != null && !getSelection().isEmpty()) {
			if (getSelectedObjects().get(0) instanceof HLTransitionEditPart) {
				HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) (getSelectedObjects().get(0));
				HLTransition t = transitionEditpart.getCastedModel();
				if (t.getKind().getValue() == HLTransitionKind.APPLYRULE
						&& HLTransitionUtil.canFire(t) && HLTransitionUtil.isValid(t) )
					return true;
			}
		}
		return false;
	}

}
