package roneditor.gef.actions.ron;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;

import roneditor.gef.commands.ron.ApplyRulesCommand;
import roneditor.gef.editparts.tree.HLPlaceTreeEditPart;
import roneditor.ui.part.RONTreeEditor;
import ronmodel.HLPlace;
import ronmodel.ObjectNet;
import ronmodel.Rule;

/**
 * Apply the {@link Rule}s of one selected {@link HLPlace} to the
 * {@link ObjectNet}s of another {@link HLPlace}
 */
public class ApplyRulesAction extends SelectionAction {

	public static final String ID = "roneditor.actions.ApplyRulesAction";

	public static final String NAME = "Apply Rules";

	public static final String DESCRIPTION = "Apply the Rules of one selected HLPlace to the Object Networks of another HLPlace";

	protected Command command;

	public ApplyRulesAction(final RONTreeEditor part) {
		super(part);
		setId(ID);
		setText(NAME);
		setDescription(DESCRIPTION);
		setToolTipText(DESCRIPTION);
	}

	@Override
	public void run() {
		if (command != null)
			execute(command);
	}

	/**
	 * Return a {@link ApplyRulesCommand} if applicable.
	 * 
	 * @return
	 */
	public Command getCommand() {
		HLPlace[] places = getCastedSelection();
		if (places != null)
			return new ApplyRulesCommand(places[0], places[1]);
		return null;
	}

	/**
	 * If there are two {@link HLPlace} selected return an casted array of them,
	 * otherwise return <code>null</code>
	 * 
	 * @return
	 */
	protected HLPlace[] getCastedSelection() {
		final StructuredSelection selection = ((StructuredSelection) getSelection());
		if (selection == null || selection.size() != 2)
			return null;

		final Object[] selectionElement = selection.toArray();

		if (!(selectionElement[0] instanceof HLPlaceTreeEditPart && selectionElement[1] instanceof HLPlaceTreeEditPart))
			return null;

		HLPlaceTreeEditPart p1 = (HLPlaceTreeEditPart) selectionElement[0];
		HLPlaceTreeEditPart p2 = (HLPlaceTreeEditPart) selectionElement[1];

		return new HLPlace[] { p1.getCastedModel(), p2.getCastedModel() };
	}

	/**
	 * The {@link SelectionAction} is enabled if the command can be executed.
	 */
	@Override
	protected boolean calculateEnabled() {
		command = getCommand();
		return command != null && command.canExecute();
	}

}
