package roneditor.gef.actions.ron;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;

import roneditor.gef.commands.ron.CreateHLPlaceCommand;
import roneditor.ui.part.RONTreeEditor;
import roneditor.ui.wizard.CreateHLPlaceWizard;
import ronmodel.HLPlaceType;

public class CreateHLPlaceAction extends SelectionAction {

	static public final String ID = "Create HLPlace";

	public CreateHLPlaceAction(final RONTreeEditor part) {
		super(part);
		setId(CreateHLPlaceAction.ID);
		setText("Create HLPlace");
		setDescription("Create a place in the RON");
		setToolTipText("Create a place in the RON");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		// create dialog
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final IWorkbench workbench = getWorkbenchPart().getSite().getWorkbenchWindow().getWorkbench();
		
		CreateHLPlaceWizard placeWizard = new CreateHLPlaceWizard();
		placeWizard.init(workbench, (IStructuredSelection) getSelection());
		WizardDialog dialog = new WizardDialog(shell, placeWizard);
		dialog.create();
		dialog.open();

		if (dialog.getReturnCode() == WizardDialog.CANCEL) {
			return;
		}

		// fill command from editpart with dialog data
		final CreateHLPlaceCommand command = getCommand();
		command.setName(placeWizard.getPlaceName());
		if (placeWizard.isRule()) {
			command.setType(HLPlaceType.RULE_LITERAL);
		} else {
			command.setType(HLPlaceType.NET_LITERAL);
		}
		execute(command);
	}

	/**
	 * Gets the command from the selected edit part which delivers a Command for
	 * the ID request since {@link #calculateEnabled()} must have returned true.
	 */
	private CreateHLPlaceCommand getCommand() {
		// HINT Die Action besorgt sich ein Command vom ausgew�hlten Editpart
		final Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		return (CreateHLPlaceCommand) ((EditPart) selection).getCommand(new Request(ID));
	}

	/**
	 * Calculates whether this action is enabled or not by checking if it
	 * understands the ID request.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		// HINT Diese Action soll nur f�r genau einen ausgew�hlten EditPart
		// angezeigt
		// werden, der auch den Request ID versteht
		if (getSelectedObjects().size() == 1 && (getSelectedObjects().get(0) instanceof EditPart)) {
			final EditPart part = (EditPart) getSelectedObjects().get(0);
			return part.understandsRequest(new Request(ID));
		}
		return false;
	}
}
