package roneditor.gef.actions.ron;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;

import roneditor.gef.commands.ron.CreateTokenCommand;
import roneditor.ui.part.RONTreeEditor;
import roneditor.ui.wizard.CreateTokenWizard;
import ronmodel.Token;

/**
 * An action for the RON editor to create tokens (i.e. low-level nets or rules)
 * on places in a RON.
 */
public class CreateTokenAction extends SelectionAction {

	static public final String ID = "Create Token";

	public CreateTokenAction(final RONTreeEditor part) {
		super(part);
		setId(CreateTokenAction.ID);
		setText("Create Token");
		setDescription("Create token on the place");
		setToolTipText("Create token on the place");
	}

	/**
	 * The Action shows an {@link CreateTokenWizard} and asks for the name of the {@link Token}. 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final IWorkbench workbench = getWorkbenchPart().getSite().getWorkbenchWindow().getWorkbench();
		
		CreateTokenWizard tokenWizard = new CreateTokenWizard();
		tokenWizard.init(workbench, (IStructuredSelection) getSelection());
		WizardDialog dialog = new WizardDialog(shell, tokenWizard);
		dialog.create();
		dialog.open();
		if (dialog.getReturnCode() == WizardDialog.CANCEL) {
			return;
		}
		
		// fill command from editpart with dialog data
		final CreateTokenCommand command = getCommand();
		command.setName(tokenWizard.getTokenName());
		execute(command);
	}

	/**
	 * Gets the command from the selected edit part which delivers a Command for
	 * the ID request since {@link #calculateEnabled()} must have returned true.
	 */
	private CreateTokenCommand getCommand() {
		final Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		return (CreateTokenCommand) ((EditPart) selection).getCommand(new Request(ID));
	}

	/**
	 * Calculates whether this action is enabled or not by checking if it
	 * understands the ID request.
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelectedObjects().size() == 1 && (getSelectedObjects().get(0) instanceof EditPart)) {
			final EditPart part = (EditPart) getSelectedObjects().get(0);
			return part.understandsRequest(new Request(ID));
		}
		return false;
	}
}
