package roneditor.gef.actions.ron;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.gef.editparts.tree.RONTreeEditPart;
import roneditor.model.HLTransitionUtil;
import roneditor.ui.dialog.FireDialog;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;


/**
 * {@link SelectionAction} for choosing a {@link ONTransition} in an {@link ObjectNet}
 * that should be fired.  
 */
public class FireDialogAction extends SelectionAction {

	public static final String ID = "roneditor.actions.FireAction";

	public static final String NAME = "Fire Dialog";

	public FireDialogAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) (getSelectedObjects().get(0));
		HLTransition hltransition = transitionEditpart.getCastedModel();

		// get first object net in the place
		HLArc arc = (HLArc) (hltransition.getIncoming().get(0));
		HLPlace place = (HLPlace) arc.getSource();

		// open dialog
		final FireDialog dialog = new FireDialog((ObjectNet) (place.getTokens().get(0)));
		dialog.open();
		if (dialog.isCanceled()) {
			return;
		}

		// fetch information from the dialog
		final ONTransition transition = dialog.getTransitionToFire();

		FireTransitionCommand command = new FireTransitionCommand(transition);
		getCommandStack().execute(command);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link RONTreeEditPart}.
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelection() != null && !getSelection().isEmpty()) {
			if (getSelectedObjects().get(0) instanceof HLTransitionEditPart) {
				HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) (getSelectedObjects().get(0));
				HLTransition t = transitionEditpart.getCastedModel();
				if (t.getKind().getValue() == HLTransitionKind.FIRE && HLTransitionUtil.canFire(t) && HLTransitionUtil.isValid(t))
					return true;
			}
		}
		return false;
	}
}
