package roneditor.gef.actions.rule;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.ui.IEditorPart;

import roneditor.gef.commands.rule.CreateFromCollectionCommand;
import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.ui.part.RulePage;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;

/**
 * Copies the object net in the {@link RHS} to the active {@link NAC} given a
 * {@link RulePage}.
 */
public class CopyRHSToNACAction extends ObjectnetCopyAction {

	static public final String ID = "Copy RHS -> NAC";

	static public final String TEXT = "Copy RHS -> NAC";

	static public final String TOOL_TIP_TEXT = "Copies the object net in the RHS to the active NAC";

	protected List<Mapping> mappings; 

	public CopyRHSToNACAction(RulePage rulePage, IEditorPart editor) {
		super(editor, rulePage);
		setId(ID);
		super.setText(TEXT);
		super.setDescription(TEXT);
		super.setToolTipText(TOOL_TIP_TEXT);
	}

	@SuppressWarnings("unchecked")
	public void run() {
		try {
			Collection copyCollection = EcoreUtil.copyAll(source.eContents());
			CompoundCommand command = new CompoundCommand();
			
			command.add(new CreateFromCollectionCommand(copyCollection, target));

			for (Object mappingObject: mappings) {
				Mapping mapping = (Mapping) mappingObject;
				ONNode sourceNode = mapping.getSource();

				// check for matching target in the copied collection 
				// (we don't have a target.getNodes() with the nodes yet!)
				ONNode targetNode = null;
				for (Object targetNodeObject: copyCollection) {
					if (! (targetNodeObject instanceof ONNode)) continue;
					targetNode = (ONNode) targetNodeObject;
					if (targetNode.getName().equals(mapping.getTarget().getName())) break;
					targetNode = null;
				}
				
				// Should not happen, but skip nodes that are not found
				if ( targetNode == null ) continue; 

				Mapping newMapping = RonmodelFactory.eINSTANCE.createMapping();
				UpdateMappingCommand updateMappingCommand = new UpdateMappingCommand();
				updateMappingCommand.setSource(sourceNode);
				updateMappingCommand.setTarget(targetNode);
				updateMappingCommand.setCodomain((Codomain)target);
				updateMappingCommand.setMapping(newMapping);
				updateMappingCommand.update();

				command.add(updateMappingCommand);
			}
			execute(command);
			
				
		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}
	}
	
	@Override
	public void update() {
		setTarget(getRulePage().getActiveNAC());
		setSource(getRulePage().getRule().getRhs());
		setMappings(getRulePage().getRule().getRhs().getMappings());
		super.update();
	}
	
	public List<Mapping> getMappings() {
		return mappings;
	}

	public void setMappings(List<Mapping> mappings) {
		this.mappings = mappings;
	}
}
