package roneditor.gef.actions.rule;

import java.util.Collection;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.rule.CreateFromCollectionCommand;
import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.ui.part.RulePage;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;

/**
 * This abstract {@link WorkbenchPartAction} handles copies from the
 * <code>target</code> to <code>source</code> {@link ObjectNet}. The
 * derived classes should implement the <code>update()</code> method and set
 * the <code>target</code> and <code>source</code> there.
 */
abstract public class ObjectnetCopyAction extends WorkbenchPartAction {

	protected ObjectNet source;

	protected ObjectNet target;

	protected RulePage rulePage;

	public ObjectnetCopyAction(IWorkbenchPart part, RulePage rulePage) {
		super(part);
		setRulePage(rulePage);
	}

	@SuppressWarnings("unchecked")
	public void run() {
		try {
			Collection copyCollection = EcoreUtil.copyAll(source.eContents());
			CompoundCommand command = new CompoundCommand();

			command.add(new CreateFromCollectionCommand(copyCollection, target));

			for (Object sourceNodeObject : source.getNodes()) {
				ONNode sourceNode = (ONNode) sourceNodeObject;

				// check for matching target in the copied collection
				// (we don't have a target.getNodes() with the nodes yet!)
				ONNode targetNode = null;
				for (Object targetNodeObject : copyCollection) {
					if (!(targetNodeObject instanceof ONNode))
						continue;
					targetNode = (ONNode) targetNodeObject;
					if (targetNode.getName().equals(sourceNode.getName()))
						break;
					targetNode = null;
				}

				// Should not happen, but skip nodes that are not found
				if (targetNode == null)
					continue;

				Mapping mapping = RonmodelFactory.eINSTANCE.createMapping();
				UpdateMappingCommand updateMappingCommand = new UpdateMappingCommand();
				updateMappingCommand.setSource(sourceNode);
				updateMappingCommand.setTarget(targetNode);
				updateMappingCommand.setCodomain((Codomain) target);
				updateMappingCommand.setMapping(mapping);
				updateMappingCommand.update();
				command.add(updateMappingCommand);

			}
			execute(command);

		} catch (Exception e) {
			System.err.println(e);
			e.printStackTrace();
		}
	}

	@Override
	protected boolean calculateEnabled() {
		return (source != null && target != null);
	}

	public ObjectNet getSource() {
		return source;
	}

	public void setSource(ObjectNet source) {
		this.source = source;
	}

	public ObjectNet getTarget() {
		return target;
	}

	public void setTarget(ObjectNet target) {
		this.target = target;
	}

	public RulePage getRulePage() {
		return rulePage;
	}

	public void setRulePage(RulePage rulePage) {
		this.rulePage = rulePage;
	}

}
