package roneditor.gef.commands.objectnet;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;

import roneditor.gef.editparts.ron.HLPlaceEditPart;
import roneditor.gef.editparts.ron.SimpleTokenEditPart;
import ronmodel.Token;

public class ChangeTokenOrderCommand extends Command {

	HLPlaceEditPart context;

	SimpleTokenEditPart child;

	SimpleTokenEditPart after;

	int newPosition;

	int oldPosition;

	EList tokens;

	/**
	 * Changes the order of {@link Token} within an {@link HLPlaceEditPart}.
	 * 
	 * @param context
	 *            the {@link HLPlaceEditPart} where the
	 *            {@link SimpleTokenEditPart} are in
	 * @param child
	 *            the child being moved
	 * @param after
	 *            null or the EditPart that should be after (or to the right of)
	 *            the child being moved
	 */
	public ChangeTokenOrderCommand(HLPlaceEditPart context, SimpleTokenEditPart child, SimpleTokenEditPart after) {
		this.context = context;
		this.child = child;
		this.after = after;
		this.tokens = context.getCastedModel().getTokens();
	}


	/**
	 * Execute the {@link Command}, so change the order within the {@link EList} of the {@link Token}
	 */
	@Override
	public void execute() {
		newPosition = tokens.size();
		if (after != null)
			newPosition = tokens.indexOf(after.getCastedModel());

		oldPosition = tokens.indexOf(child.getCastedModel());

		// we shift the index when we move down (as the item above is beeing
		// removed)
		if (newPosition > oldPosition)
			newPosition--;

		tokens.move(newPosition, oldPosition);
	}

	@Override
	public void undo() {
		tokens.move(oldPosition, newPosition);
	}

	@Override
	public void redo() {
		tokens.move(newPosition, oldPosition);
	}

}
