package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Command for connecting an {@link ONPlace} with an {@link ONTransition}.
 */
public class ConnectPlaceTransitionCommand extends Command {
	private static final String LABEL = "ConnectPlaceTransitionCommand";

	private static final String DESCRIPTION = "ONArc: Connect a place with a transition";

	protected ONPlace source;

	protected ONTransition target;

	protected ONArc arc;

	private String name = "";

	/**
	 * Constructor.
	 */
	public ConnectPlaceTransitionCommand() {
		super(LABEL);
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target)
			return false;
		else
			return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		ObjectNet objectnet = target.getObjectNet();
		arc.setObjectNet(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
		arc.setName(name);
	}

	/**
	 * Returns the command nameLabel.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setObjectNet(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the ONArc to be used.
	 * 
	 * @param link
	 */
	public void setArc(ONArc link) {
		this.arc = link;
	}

	/**
	 * Set the given {@link ONPlace} as source.
	 * 
	 * @param source
	 */
	public void setSource(ONPlace source) {
		this.source = source;
	}

	/**
	 * Set the given {@link ONTransition} as target.
	 * 
	 * @param target
	 */
	public void setTarget(ONTransition target) {
		this.target = target;
	}

	/**
	 * @return The {@link ONArc} used.
	 */
	public ONArc getArc() {
		return arc;
	}

	/**
	 * @return The {@link ONPlace} used as source.
	 */
	public ONPlace getSource() {
		return source;
	}

	/**
	 * @return The {@link ONTransition} used as target.
	 */
	public ONTransition getTarget() {
		return target;
	}

	/**
	 * @return The name of the {@link ONArc}
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the name of the {@link ONArc} to be used.
	 * 
	 * @param name
	 *            The name of the {@link ONArc}
	 */
	public void setName(String name) {
		this.name = name;
	}
}
