package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Command for connecting an {@link ONTransition} with an {@link ONPlace}.
 */
public class ConnectTransitionPlaceCommand extends Command {

	public static final String LABEL = "ConnectTransitionPlaceCommand";

	public static final String DESCRIPTION = "ONArc: Connect a transition with a place";

	protected ONTransition source;

	protected ONPlace target;

	protected ONArc arc;

	private String name = "";

	/**
	 * Constructor.
	 */
	public ConnectTransitionPlaceCommand() {
		super(LABEL);
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target)
			return false;
		else
			return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		ObjectNet objectnet = target.getObjectNet();
		arc.setObjectNet(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
		arc.setName(name);
	}

	/**
	 * Returns the command nameLabel.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setObjectNet(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the ONArc to be used.
	 * 
	 * @param arc
	 */
	public void setArc(ONArc arc) {
		this.arc = arc;
	}

	/**
	 * Set the given {@link ONTransition} as source.
	 * 
	 * @param source
	 */
	public void setSource(ONTransition source) {
		this.source = source;
	}

	/**
	 * Set the given {@link ONPlace} as target.
	 * 
	 * @param target
	 */
	public void setTarget(ONPlace target) {
		this.target = target;
	}

	/**
	 * @return The {@link ONArc} used.
	 */
	public ONArc getArc() {
		return arc;
	}

	/**
	 * @return The {@link ONTransition} used as source.
	 */
	public ONTransition getSource() {
		return source;
	}

	/**
	 * @return The {@link ONPlace} used as target.
	 */
	public ONPlace getTarget() {
		return target;
	}

	/**
	 * @return The name of the ONArc
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the name of the ONArc to be used.
	 * 
	 * @param name
	 *            The name of the ONArc
	 */
	public void setName(String name) {
		this.name = name;
	}
}
