package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command for deleting a {@link ONArc} between two {@link ONNode}s.
 */
public class DeleteArcCommand extends Command {
	
	public static final String LABEL = "DeleteArcCommand";

	private ONArc arc;

	private ONNode source;

	private ONNode target;

	private ObjectNet network;

	public DeleteArcCommand() {
		super(LABEL);
	}

	public DeleteArcCommand(ONArc arc) {
		super();
		setONArc(arc);
	}

	/**
	 * Execute the command.
	 */
	public void execute() {
		source = arc.getSource();
		target = arc.getTarget();
		network = arc.getObjectNet();
		arc.setObjectNet(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setObjectNet(network);
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * @return the network
	 */
	public ObjectNet getObjectNet() {
		return network;
	}

	/**
	 * @param network
	 *            the network to set
	 */
	public void setObjectNet(ObjectNet network) {
		this.network = network;
	}

	/**
	 * @return the arc
	 */
	public ONArc getONArc() {
		return arc;
	}

	/**
	 * @param arc
	 *            the arc to set
	 */
	public void setONArc(ONArc arc) {
		this.arc = arc;
	}

	/**
	 * @return the source
	 */
	public ONNode getSource() {
		return source;
	}

	/**
	 * @param source
	 *            the source to set
	 */
	public void setSource(ONNode source) {
		this.source = source;
	}

	/**
	 * @return the target
	 */
	public ONNode getTarget() {
		return target;
	}

	/**
	 * @param target
	 *            the target to set
	 */
	public void setTarget(ONNode target) {
		this.target = target;
	}

}
