package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * The <code>ReconnectCommand</code> reconnects the given {@link ONPlace} and
 * {@link ONTransition} or vice versa.
 */
public class ReconnectCommand extends Command {
	private static final String LABEL = "ReconnectCommand";

	private static final String DESCRIPTION = "Command to reconnect an ONPlace / ONTransition";

	private ONNode oldSource;

	private ONNode oldTarget;

	private ONNode source;

	private ONNode target;

	private ONArc arc;

	/**
	 * Constructor.
	 */
	public ReconnectCommand() {
		super(LABEL);
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		oldSource = arc.getSource();
		oldTarget = arc.getTarget();
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Returns the command label.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setSource(oldSource);
		arc.setTarget(oldTarget);
	}

	/**
	 * Set the arc
	 * 
	 * @param arc
	 */
	public void setArc(ONArc arc) {
		this.arc = arc;
	}

	/**
	 * Return the connection as ONArc.
	 * 
	 * @return The ONArc connection the ONNodes
	 */
	public ONArc getArc() {
		return arc;
	}

	/**
	 * Get the source node.
	 * 
	 * @return the source of the connection as ONNode
	 */
	public ONNode getSource() {
		return source;
	}

	/**
	 * Set the source node of the connection
	 * 
	 * @param source
	 */
	public void setSource(ONNode source) {
		this.source = source;
	}

	/**
	 * Get the target node of the connection.
	 * 
	 * @return target
	 */
	public ONNode getTarget() {
		return target;
	}

	/**
	 * Set the target node of the connection-
	 * 
	 * @param target
	 */
	public void setTarget(ONNode target) {
		this.target = target;
	}

}
