package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.NamedElement;

/**
 * The <code>RenameNamedElementCommand</code> is used for changing the name of an
 * {@link NamedElement}.
 */
public class RenameNamedElementCommand extends Command {
	private NamedElement namedElement;

	private String oldName = "";

	private String newName = "";

	/**
	 * Create a {@link RenameNamedElementCommand}. You have to provide the
	 * {@link NamedElement} and the new name manually.
	 */
	public RenameNamedElementCommand() {
		setLabel("Rename NamedElement");
	}

	/**
	 * Create a {@link RenameNamedElementCommand} that changes the name of a
	 * {@link NamedElement}.
	 * 
	 * @param namedElement
	 *            The ONNode to change
	 * @param newName
	 *            The new name for the ONNode
	 */
	public RenameNamedElementCommand(NamedElement namedElement, String newName) {
		this();
		setNamedElement(namedElement);
		setNewName(newName);
	}

	/**
	 * Execute the {@link RenameNamedElementCommand}, hence set the new name
	 * for the {@link NamedElement}.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		oldName = namedElement.getName();
		namedElement.setName(newName);
	}

	/**
	 * Revert changes done by the command. Set the name to the old name.
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		namedElement.setName(oldName);
	}

	/**
	 * Redo the changes done by the command. Set the name to the new name.
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		namedElement.setName(newName);
	}

	/**
	 * Return the {@link NamedElement} of the command.
	 * 
	 * @return The {@link NamedElement}
	 */
	public NamedElement getNamedElement() {
		return namedElement;
	}

	/**
	 * Set the Node for the command.
	 * 
	 * @param namedElement
	 *            the {@link NamedElement} to set
	 */
	public void setNamedElement(NamedElement namedElement) {
		this.namedElement = namedElement;
	}

	/**
	 * Return the new name
	 * 
	 * @return the newName
	 */
	public String getNewName() {
		return newName;
	}

	/**
	 * The new Name to be used.
	 * 
	 * @param newName
	 *            the newName to set
	 */
	public void setNewName(String newName) {
		this.newName = newName;
	}

}
