package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;

/**
 * Command to change the weight of an {@link ONArc}
 */
public class SetArcWeightCommand extends Command {
	int oldWeight;

	int newWeight;

	ONArc arc;

	/**
	 * Constructor needs the {@link ONArc} object to update and the new weight.
	 * 
	 * @param arc
	 * @param newWeight
	 */
	public SetArcWeightCommand(ONArc arc, int newWeight) {
		this.arc = arc;
		this.newWeight = newWeight;
		this.oldWeight = arc.getWeight();
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		arc.setWeight(newWeight);
	}

	/**
	 * Revert changes done by the command. Set the weight to the old weight.
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		arc.setWeight(oldWeight);
	}

	/**
	 * Redo the changes done by the command. Set the weight to the new weight.
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		arc.setWeight(newWeight);
	}
}
