package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;

/**
 * Command for connecting an {@link ONPlace} with an {@link ONTransition}.
 */
public class ConnectHLPlaceTransitionCommand extends Command {
	private static final String LABEL = "ConnectPlaceTransitionCommand";

	private static final String DESCRIPTION = "HLArc: Connect a hlplace with a hltransition";

	protected HLPlace source;

	protected HLTransition target;

	protected HLArc arc;

	private String name = "";

	/**
	 * Constructor.
	 */
	public ConnectHLPlaceTransitionCommand() {
		super(LABEL);
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target)
			return false;
		else
			return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		RON objectnet = target.getRon();
		arc.setRon(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
		arc.setName(name);
	}

	/**
	 * Returns the command nameLabel.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setRon(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the ONArc to be used.
	 * 
	 * @param link
	 */
	public void setArc(HLArc link) {
		this.arc = link;
	}

	/**
	 * Set the given {@link HLPlace} as source.
	 * 
	 * @param source
	 */
	public void setSource(HLPlace source) {
		this.source = source;
	}

	/**
	 * Set the given {@link HLTransition} as target.
	 * 
	 * @param target
	 */
	public void setTarget(HLTransition target) {
		this.target = target;
	}

	/**
	 * @return The {@link HLArc} used.
	 */
	public HLArc getArc() {
		return arc;
	}

	/**
	 * @return The {@link HLPlace} used as source.
	 */
	public HLPlace getSource() {
		return source;
	}

	/**
	 * @return The {@link HLTransition} used as target.
	 */
	public HLTransition getTarget() {
		return target;
	}

	/**
	 * @return The name of the {@link HLArc}
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the name of the {@link HLArc} to be used.
	 * 
	 * @param name
	 *            The name of the {@link HLArc}
	 */
	public void setName(String name) {
		this.name = name;
	}
}
