package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;

/**
 * Command for connecting an {@link ONTransition} with an {@link ONPlace}.
 */
public class ConnectHLTransitionPlaceCommand extends Command {

	public static final String LABEL = "ConnectHLTransitionPlaceCommand";

	public static final String DESCRIPTION = "HLArc: Connect a transition with a place";

	protected HLTransition source;

	protected HLPlace target;

	protected HLArc arc;

	private String name = "";

	/**
	 * Constructor.
	 */
	public ConnectHLTransitionPlaceCommand() {
		super(LABEL);
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target)
			return false;
		else
			return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		RON objectnet = target.getRon();
		arc.setRon(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
		arc.setName(name);
	}

	/**
	 * Returns the command nameLabel.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setRon(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the ONArc to be used.
	 * 
	 * @param arc
	 */
	public void setArc(HLArc arc) {
		this.arc = arc;
	}

	/**
	 * Set the given {@link HLTransition} as source.
	 * 
	 * @param source
	 */
	public void setSource(HLTransition source) {
		this.source = source;
	}

	/**
	 * Set the given {@link HLPlace} as target.
	 * 
	 * @param target
	 */
	public void setTarget(HLPlace target) {
		this.target = target;
	}

	/**
	 * @return The {@link HLArc} used.
	 */
	public HLArc getArc() {
		return arc;
	}

	/**
	 * @return The {@link HLTransition} used as source.
	 */
	public HLTransition getSource() {
		return source;
	}

	/**
	 * @return The {@link HLPlace} used as target.
	 */
	public HLPlace getTarget() {
		return target;
	}

	/**
	 * @return The name of the HLArc
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set the name of the HLArc to be used.
	 * 
	 * @param name
	 *            The name of the HLArc
	 */
	public void setName(String name) {
		this.name = name;
	}
}
