/**
 * 
 */
package roneditor.gef.commands.ron;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.gef.commands.Command;

import roneditor.model.RONModelManager;
import ronmodel.NAC;
import ronmodel.Rule;

/**
 * Simple Command to add a {@link NAC} to an existing {@link Rule}. 
 */
public class CreateNACCommand extends Command {
	public static final String LABEL = "Create NAC";
	
	public static final String NACPrefix = "nac "; 

	Rule rule;

	String name;

	NAC nac;

	public CreateNACCommand(Rule rule) {
		super(LABEL);
		this.rule = rule;
	}

	@Override
	public boolean canExecute() {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void execute() {
		nac = RONModelManager.getFactory().createNAC();
		// compute highest free number for (autogenerated) nac names
		int freeNac = 1;
		List<NAC> nacs = rule.getNacs();
		for ( NAC nac : nacs ) {
			String name = nac.getName();
			if (name != null ) {
				Pattern p = Pattern.compile(NACPrefix+"(\\d*)");
				Matcher m = p.matcher(name);
				if ( m.matches() ) {
					int i = Integer.parseInt(m.group(1));
					if ( freeNac <= i ) freeNac = i+1;
				}
			}
		}
		nac.setName(NACPrefix+freeNac);
		nac.setRule(rule);
	}

	@Override
	public void redo() {
		nac.setRule(rule);
	}

	@Override
	public void undo() {
		nac.setRule(null);
	}
	
	public NAC getNac() {
		return nac;
	}

}
