package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RON;

/**
 * Command for deleting a {@link HLArc} between two {@link HLNode}s.
 */
public class DeleteHLArcCommand extends Command {
	
	public static final String LABEL = "DeleteArcCommand";

	private HLArc arc;

	private HLNode source;

	private HLNode target;

	private RON network;

	public DeleteHLArcCommand() {
		super(LABEL);
	}

	public DeleteHLArcCommand(HLArc arc) {
		super();
		setHLArc(arc);
	}

	/**
	 * Execute the command.
	 */
	public void execute() {
		source = arc.getSource();
		target = arc.getTarget();
		network = arc.getRon();
		arc.setRon(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setRon(network);
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * @return the network
	 */
	public RON getObjectNet() {
		return network;
	}

	/**
	 * @param network
	 *            the network to set
	 */
	public void setObjectNet(RON network) {
		this.network = network;
	}

	/**
	 * @return the arc
	 */
	public HLArc getHLArc() {
		return arc;
	}

	/**
	 * @param arc
	 *            the arc to set
	 */
	public void setHLArc(HLArc arc) {
		this.arc = arc;
	}

	/**
	 * @return the source
	 */
	public HLNode getSource() {
		return source;
	}

	/**
	 * @param source
	 *            the source to set
	 */
	public void setSource(HLNode source) {
		this.source = source;
	}

	/**
	 * @return the target
	 */
	public HLNode getTarget() {
		return target;
	}

	/**
	 * @param target
	 *            the target to set
	 */
	public void setTarget(HLNode target) {
		this.target = target;
	}

}
