package roneditor.gef.commands.ron;

import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.ONNode;
import ronmodel.RON;

/**
 * Command for removing a {@link HLNode} from its parent {@link RON}. This is
 * undoable.
 */
public class DeleteHLNodeCommand extends CompoundCommand {

	public static final String LABEL = "DeleteHLNodeCommand";

	/**
	 * The inner class <code>DeleteONNodeCommand</code> is part of the
	 * compound and deletes the given {@link ONNode} with no further checks.
	 */
	private class DeletePureHLNodeCommand extends Command {
		public static final String LABEL = "DeletePureHLNodeCommand";

		private HLNode node;

		private RON ron;

		public DeletePureHLNodeCommand() {
			super(LABEL);
		}

		public DeletePureHLNodeCommand(HLNode node) {
			super(LABEL);
			this.node = node;
			this.ron = node.getRon();
		}

		@Override
		public boolean canExecute() {
			return (node != null);
		}

		/**
		 * Executes the command by first hiding all views which display the
		 * element that will be removed. Then the child gets removed from its
		 * parent. The container is stored for undoing purposes.
		 * 
		 * @see org.eclipse.gef.commands.Command#execute()
		 */
		@Override
		public void execute() {
			ron = node.getRon();
			node.setRon(null);
		}

		@Override
		public void redo() {
			node.setRon(null);
		}

		@Override
		public void undo() {
			node.setRon(ron);
		}
	}

	@SuppressWarnings("unchecked")
	public DeleteHLNodeCommand(HLNode node) {
		super(LABEL);

		// Delete ONArcs
		Vector<HLArc> incomingArcs = new Vector<HLArc>(node.getIncoming());
		for (HLArc link : incomingArcs) {
			add(new DeleteHLArcCommand(link));
		}
		Vector<HLArc> outgoingArcs = new Vector<HLArc>(node.getOutgoing());
		for (HLArc link : outgoingArcs) {
			add(new DeleteHLArcCommand(link));
		}

		// Delete HLNode (a "pure" one) itself
		add(new DeletePureHLNodeCommand(node));
	}
}
