/**
 * 
 */
package roneditor.gef.commands.ron;

import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.commands.objectnet.DeleteNodeCommand;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Rule;

/**
 * {@link CompoundCommand} to delete a {@link NAC} from a {@link Rule}.
 */
public class DeleteNACCommand extends CompoundCommand {

	public static final String LABEL = "DeleteNACCommand";

	protected NAC nac;

	protected ObjectNet objectNet;

	/**
	 * Private inner class that deletes a {@link NAC}, hence remove it from the
	 * Model by setting removing the {@link Rule} association.
	 */
	private class DeleteNACOnlyCommand extends Command {
		private static final String LABEL = "DeleteNACOnlyCommand";

		Rule rule;

		String name;

		NAC nac;

		public DeleteNACOnlyCommand(NAC nac) {
			super(LABEL);
			this.nac = nac;
		}

		@Override
		public boolean canExecute() {
			return true;
		}

		@Override
		public void execute() {
			rule = nac.getRule();
			nac.setRule(null);
		}

		@Override
		public void redo() {
			nac.setRule(null);
		}

		@Override
		public void undo() {
			nac.setRule(rule);
		}
	}

	/**
	 * The command will delete a {@link NAC} and all {@link ONNode} in it. As
	 * the {@link DeleteNodeCommand} also deletes all attached {@link ONArc} and
	 * {@link Mapping} the consistency on undo is ensured.
	 * 
	 * @param nac
	 *            The {@link NAC} to delete.
	 */
	@SuppressWarnings("unchecked")
	public DeleteNACCommand(NAC nac) {
		super(LABEL);
		this.nac = nac;
		List<ONNode> nodes = nac.getNodes();
		for (ONNode node : nodes) {
			add(new DeleteNodeCommand(node));
		}
		add(new DeleteNACOnlyCommand(nac));
	}

}
