package roneditor.gef.commands.ron;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.commands.rule.ApplyRuleCommand;
import roneditor.model.HLTransitionUtil;
import roneditor.model.ObjectNetUtil;
import roneditor.model.RONModelManager;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * Perform <code>APPLYRULE</code> operation on the {@link HLTransition}.
 */
public class HLTransitionApplyRuleCommand extends CompoundCommand {
	ApplyRuleCommand applyRuleCommand;

	@SuppressWarnings("unchecked")
	public HLTransitionApplyRuleCommand(HLTransition transition) {
		// Collect incoming NET/RULE HLPlaces

		// Collect all NET tokens to move/combine
		// also consider multiple tokens from the same place
		List<ObjectNet> tokens = HLTransitionUtil.collectSourceNetTokens(transition);
		ObjectNet unionObjectNet = RONModelManager.getFactory().createObjectNet();
		ObjectNetUtil.unionIntoObjectNet(tokens, unionObjectNet);

		// apply rule to unionObjectNet
		List<HLPlace> incomingRulePlaces = HLTransitionUtil.getIncomingRulePlaces(transition);
		Token rule = (Token) incomingRulePlaces.get(0).getTokens().get(0);

		applyRuleCommand = new ApplyRuleCommand(unionObjectNet, rule);
		add(applyRuleCommand);

		// Copy rule to the new outgoing tokens on HLPlace.RULE
		List<HLPlace> outgoingRulePlaces = HLTransitionUtil.getOutgoingRulePlaces(transition);
		for (HLPlace place : outgoingRulePlaces) {
			add(new CopyTokenToHLPlaceCommand(rule, place));
		}

		// remove rule from the HLPlace of Type RULE
		add(new RemoveTokenFromHLPlaceCommand(rule));

		// Copy unionObjectNet to the new outgoing tokens on HLPlace.NET
		List<HLPlace> outgoingNetPlaces = HLTransitionUtil.getOutgoingNetPlaces(transition);
		for (HLPlace place : outgoingNetPlaces) {
			add(new CopyTokenToHLPlaceCommand((Token) unionObjectNet, place));
		}

		// remove tokens/ObjectNets from place
		for (ObjectNet t : tokens) {
			add(new RemoveTokenFromHLPlaceCommand(t));
		}

	}

	public String getLastErrorMessage() {
		return applyRuleCommand.getLastErrorMessage();
	}
	
}
