package roneditor.gef.commands.ron;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.model.ONTransitionUtil;
import roneditor.ui.wizard.ChooseONTransitionWizard;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * 
 * @author seb
 * implements HLFireTransition :)
 */
public class HLTransitionFireCommand extends CompoundCommand {

	public static final String LABEL = "HLTransitionFireCommand";
	
	
	
	@SuppressWarnings("unchecked")
	public HLTransitionFireCommand(HLTransition transition) {
		super(LABEL);
		
		
		// test if only one NET place is in pre and only NETs are post scope
		if (transition.getIncoming().size() != 1)
			return;
		if (transition.getOutgoing().size() < 1)
			return;
		HLArc a1 = (HLArc) transition.getIncoming().get(0);
		HLPlace p = (HLPlace) a1.getSource();
		if (p == null) return;
		if (p.getType() == HLPlaceType.RULE_LITERAL)
			return;
		
		List<HLPlace> outgoingNetPlaces = new LinkedList<HLPlace>();
		for (Object t : transition.getOutgoing()){
			HLPlace pl = (HLPlace) ((HLArc)t).getTarget();
			if (pl == null) return;
			if (pl.getType() == HLPlaceType.RULE_LITERAL)
				return;
			else outgoingNetPlaces.add(pl);
		}
		
		if (p.getTokens().size() > 0 ) {
			Token token = (Token) p.getTokens().get(0);
			
			LinkedList<ONTransition> activeTransitions = new LinkedList<ONTransition>();
			
			for (Object nodeObject : ((ObjectNet) token).getNodes()) {
				ONNode node = (ONNode) nodeObject;
				if (node instanceof ONPlace)
					continue;
				if (ONTransitionUtil.transitionCanFire((ONTransition) node)) {
					activeTransitions.add((ONTransition)node);
				}
			}
			
			
			ONTransition ontransition = null;

			// No active transition
			if ( activeTransitions.size() == 0 ) 
				return;
			
			
			// Only one active transition
			if ( activeTransitions.size() == 1 )
				ontransition = activeTransitions.get(0);
			else {
				// Ask which transition to fire...
				IWorkbench workbench = PlatformUI.getWorkbench();
				Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

				ChooseONTransitionWizard placeWizard = new ChooseONTransitionWizard(activeTransitions);
				placeWizard.init(workbench, null);
				
				WizardDialog dialog = new WizardDialog(shell, placeWizard);
				dialog.create();
				dialog.open();

				if (dialog.getReturnCode() == WizardDialog.CANCEL) {
					return;
				}
				
				ontransition = placeWizard.getSelectedTransition();
			}
			
			// Fire the transition
			add(new FireTransitionCommand(ontransition));
			
			// Copy token to the new outgoing tokens on HLPlace.NET
			for (HLPlace place : outgoingNetPlaces) {
				add(new CopyTokenToHLPlaceCommand(token, place));
			}
			// remove first token/ObjectNet from place
			add(new RemoveTokenFromHLPlaceCommand(token));
			
			

			
		}
	}
	
}
