package roneditor.gef.commands.ron;

import java.util.HashMap;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * Move multiple {@link Token} in a {@link List} to the given {@link HLPlace}.
 * The given {@link List} should contain only {@link Token} that match the type
 * of the {@link HLPlace}, otherwise the {@link Command} will not be executed.
 */
public class MoveMultipleTokenCommand extends Command {
	HLPlace place;

	HashMap<Token, HLPlace> oldPlaceList;

	List tokenList;

	@SuppressWarnings("unchecked")
	public MoveMultipleTokenCommand(List tokenList, HLPlace place) {
		this.tokenList = tokenList;
		this.place = place;
	}

	/**
	 * Return true if every member of the tokenList is a {@link Token} that
	 * matches the {@link HLPlaceType}.
	 */
	@Override
	public boolean canExecute() {
		for (Object o : tokenList) {
			if (place.getType().equals(HLPlaceType.NET_LITERAL)) {
				if (!(o instanceof ObjectNet))
					return false;
			} else {
				if (!(o instanceof Rule))
					return false;
			}
		}
		return true;
	}

	/**
	 * Store old places in a {@link HashMap} and update the {@link Token}s to
	 * the current {@link HLPlace}
	 */
	@Override
	public void execute() {
		oldPlaceList = new HashMap<Token, HLPlace>();
		for (Object o : tokenList) {
			Token token = (Token) o;
			oldPlaceList.put(token, token.getPlace());
			token.setPlace(place);
		}
	}

	/**
	 * Revert the {@link HLPlace}s of the {@link Token}s by querying the
	 * {@link HashMap}.
	 */
	@Override
	public void undo() {
		for (Object o : tokenList) {
			Token token = (Token) o;
			token.setPlace(oldPlaceList.get(token));
		}
	}

	/**
	 * Put the {@link Token}s again on the current {@link HLPlace}.
	 */
	@Override
	public void redo() {
		for (Object o : tokenList) {
			Token token = (Token) o;
			token.setPlace(place);
		}
	}

}
