package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * The <code>ReconnectCommand</code> reconnects the given {@link ONPlace} and
 * {@link ONTransition} or vice versa.
 */
public class ReconnectHLNodeCommand extends Command {
	private static final String LABEL = "ReconnectCommand";

	private static final String DESCRIPTION = "Command to reconnect an HLPlace / HLTransition";

	private HLNode oldSource;

	private HLNode oldTarget;

	private HLNode source;

	private HLNode target;

	private HLArc arc;

	/**
	 * Constructor.
	 */
	public ReconnectHLNodeCommand() {
		super(LABEL);
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		oldSource = arc.getSource();
		oldTarget = arc.getTarget();
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Returns the command label.
	 */
	public String getLabel() {
		return DESCRIPTION;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		arc.setSource(oldSource);
		arc.setTarget(oldTarget);
	}

	/**
	 * Set the arc
	 * 
	 * @param arc
	 */
	public void setArc(HLArc arc) {
		this.arc = arc;
	}

	/**
	 * Return the connection as ONArc.
	 * 
	 * @return The ONArc connection the ONNodes
	 */
	public HLArc getArc() {
		return arc;
	}

	/**
	 * Get the source node.
	 * 
	 * @return the source of the connection as HLNode
	 */
	public HLNode getSource() {
		return source;
	}

	/**
	 * Set the source node of the connection
	 * 
	 * @param source
	 */
	public void setSource(HLNode source) {
		this.source = source;
	}

	/**
	 * Get the target node of the connection.
	 * 
	 * @return target
	 */
	public HLNode getTarget() {
		return target;
	}

	/**
	 * Set the target node of the connection-
	 * 
	 * @param target
	 */
	public void setTarget(HLNode target) {
		this.target = target;
	}

}
