package roneditor.gef.commands.ron;

import org.eclipse.draw2d.geometry.Point;

import ronmodel.HLNode;

/**
 * Change the location of a graphical object.
 */
public class SetConstraintCommand extends org.eclipse.gef.commands.Command {
	private static final String LABEL = "SetConstraintCommand";

	private Point newPosition;

	private Point oldPosition;

	private Point oldSize = null;

	private Point newSize = null;

	private HLNode node;

	public void execute() {
		oldPosition = new Point(node.getX(), node.getY());

		oldSize = new Point(node.getWidth(), node.getHeight());

		// default: don't do anything
		if (newSize == null)
			newSize = oldSize;
		if (newPosition == null)
			newPosition = oldPosition;

		node.setX(newPosition.x);
		node.setY(newPosition.y);

		node.setWidth(newSize.x);
		node.setHeight(newSize.y);
	}

	public String getLabel() {
		return LABEL;
	}

	public void redo() {
		node.setX(newPosition.x);
		node.setY(newPosition.y);
	}

	public void undo() {
		node.setX(oldPosition.x);
		node.setY(oldPosition.y);
	}

	/**
	 * @return the newPosition
	 */
	public Point getNewPosition() {
		return newPosition;
	}

	/**
	 * @param newPosition
	 *            the newPosition to set
	 */
	public void setNewPosition(Point newPosition) {
		this.newPosition = newPosition;
	}

	/**
	 * @return the node
	 */
	public HLNode getHLNode() {
		return node;
	}

	/**
	 * @param node
	 *            the node to set
	 */
	public void setHLNode(HLNode node) {
		this.node = node;
	}

	/**
	 * @return the oldSize
	 */
	public Point getOldSize() {
		return oldSize;
	}

	/**
	 * @param oldSize
	 *            the oldSize to set
	 */
	public void setOldSize(Point oldSize) {
		this.oldSize = oldSize;
	}

	/**
	 * @return the newSize
	 */
	public Point getNewSize() {
		return newSize;
	}

	/**
	 * @param newSize
	 *            the newSize to set
	 */
	public void setNewSize(Point newSize) {
		this.newSize = newSize;
	}

}
