package roneditor.gef.commands.ron;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * pseudo command that unions {@link ObjectNet}s into <i>one</i>
 * {@link ObjectNet}.
 */
public class UnionObjectNetsIntoOneCommand extends Command {
	ObjectNet target;

	String targetName;

	Collection allTokenContent = null;
	List<ObjectNet> source = null;

	@SuppressWarnings("unchecked")
	public UnionObjectNetsIntoOneCommand(List<ObjectNet> source,
			ObjectNet target) {
		this.target = target;
		this.source = source;
	}
	
	@Override
	public void execute() {
		for (Token net : source) {

			if (allTokenContent == null) {
				allTokenContent = EcoreUtil.copyAll(net.eContents());
				targetName = net.getName();
			} else
				allTokenContent.addAll(EcoreUtil.copyAll(net.eContents()));
		}

		redo();
	}

	public void undo() {
		for (Object object : allTokenContent) {
			if (object instanceof ONNode) {
				((ONNode) object).setObjectNet(null);
			} else if (object instanceof ONArc) {
				((ONArc) object).setObjectNet(null);
			}
		}
		target.setPlace(null);
	}

	public void redo() {
		target.setName(targetName);
		target.setPlace(null);
		for (Object object : allTokenContent) {
			if (object instanceof ONNode) {
				((ONNode) object).setObjectNet(target);
			} else if (object instanceof ONArc) {
				((ONArc) object).setObjectNet(target);
			}
		}
	}
}
