package roneditor.gef.commands.rule;

import java.util.Collection;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * {@link Command} to add {@link ONNode} and {@link ONArc} stored in a
 * {@link Collection} to an {@link ObjectNet}.
 */
public class CreateFromCollectionCommand extends Command {
	public static final String LABEL = "CreateFromCollection";

	protected ObjectNet objectNet;

	protected Collection collection;

	public CreateFromCollectionCommand(Collection collection,
			ObjectNet objectnet) {
		super(LABEL);
		setObjectNet(objectnet);
		setCollection(collection);
	}

	public void execute() {
		for (Object object : collection) {
			if (object instanceof ONNode) {
				((ONNode) object).setObjectNet(objectNet);
			} else if (object instanceof ONArc) {
				((ONArc) object).setObjectNet(objectNet);
			}
		}
	}

	public void undo() {
		for (Object object : collection) {
			if (object instanceof ONNode) {
				((ONNode) object).setObjectNet(null);
			} else if (object instanceof ONArc) {
				((ONArc) object).setObjectNet(null);
			}
		}
	}

	public void redo() {
		execute();
	}

	public Collection getCollection() {
		return collection;
	}

	public void setCollection(Collection collection) {
		this.collection = collection;
	}

	public ObjectNet getObjectNet() {
		return objectNet;
	}

	public void setObjectNet(ObjectNet objectNet) {
		this.objectNet = objectNet;
	}

}
