/**
 * 
 */
package roneditor.gef.commands.rule;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.commands.Command;

import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * {@link Command} to create a {@link Mapping} between two {@link ONNode}. The
 * <code>CreateMappingCommand</code> needs a {@link ONNode} as source and
 * target, and a {@link Codomain} to refer to. The {@link Command} when executed
 * will modify the {@link Mapping} accordingly.
 * 
 * This {@link Command} should not be used, use {@link UpdateMappingCommand}
 * instead.
 */
public class CreateMappingCommand extends Command {

	public static final String LABEL = "CreateMappingCommand";

	public static final String DESCRIPTION = "Create a mapping between Nodes";

	Mapping mapping;

	Codomain codomain;

	ONNode source;

	ONNode target;

	ONNode oldSource;

	ONNode oldTarget;

	Codomain oldCodomain;

	Mapping oldMapping;

	public CreateMappingCommand(Mapping mapping, Codomain codomain, ONNode source, ONNode target) {
		super(LABEL);
		this.mapping = mapping;
		this.codomain = codomain;
		this.source = source;
		this.target = target;
	}

	@Override
	public boolean canExecute() {
		if (source == null || target == null || mapping == null || source == target || codomain == null)
			return false;

		if (!(source.getObjectNet() instanceof LHS))
			return false;

		if (source.getObjectNet() == target.getObjectNet())
			return false;

		if (target.getClass() != source.getClass())
			return false;

		return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		mapping.setCodomain(codomain);
		mapping.setSource(source);
		mapping.setTarget(target);
		Notification notification = new NotificationImpl(0, true, false);
		if (source != null)
			source.eNotify(notification);
		if (target != null)
			target.eNotify(notification);
	}

	public void undo() {
		mapping.setCodomain(null);
		mapping.setSource(null);
		mapping.setTarget(null);
		Notification notification = new NotificationImpl(0, true, false);
		if (source != null)
			source.eNotify(notification);
		if (target != null)
			target.eNotify(notification);
	}

	public void redo() {
		execute();
	}

}
