/**
 * 
 */
package roneditor.gef.commands.rule;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.commands.Command;

import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * {@link Command} to delete a {@link Mapping} between two {@link ONNode}. The
 * <code>DeleteMappingCommand</code> needs a {@link ONNode} as source and
 * target, and a {@link Codomain} to refer to. The {@link Command} when executed
 * will remove the {@link Mapping} accordingly.
 * 
 * This {@link Command} should not be used, use {@link UpdateMappingCommand} instead.
 */
public class DeleteMappingCommand extends Command {

	public static final String LABEL = "CreateMappingCommand";

	public static final String DESCRIPTION = "Create a mapping between Nodes";

	Mapping mapping;

	Codomain codomain;

	ONNode source;

	ONNode target;

	public DeleteMappingCommand() {
		super(DeleteMappingCommand.LABEL);
	}
	
	public DeleteMappingCommand(Mapping mapping) {
		super(DeleteMappingCommand.LABEL);
		setMapping(mapping);
	}


	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	@Override
	public boolean canExecute() {
		if (mapping == null)
			return false;
		return true;
	}

	public void execute() {
		source = mapping.getSource();
		target = mapping.getTarget();
		codomain = mapping.getCodomain();

		mapping.setSource(null);
		mapping.setTarget(null);
		mapping.setCodomain(null);

		// notify source and target
		Notification notification = new NotificationImpl(0, true, false);
		if (source != null)
			source.eNotify(notification);
		if (target != null)
			target.eNotify(notification);
	}

	public void undo() {
		mapping.setCodomain(codomain);
		mapping.setSource(source);
		mapping.setTarget(target);

		// notify source and target
		Notification notification = new NotificationImpl(0, true, false);
		if (source != null)
			source.eNotify(notification);
		if (target != null)
			target.eNotify(notification);
	}

	public void redo() {
		execute();
	}
}
