package roneditor.gef.commands.rule;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.commands.CompoundCommand;

import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * {@link CompoundCommand} to dynamically add, change and delete {@link Mapping}s
 * between two {@link ONNode}s. The <code>UpdateMappingCommand</code> needs a
 * {@link ONNode} as source and target, and a {@link Codomain} to refer to. It
 * will ensure the injectivity of the {@link Mapping}s hence, it will
 * automatically remove any already existing {@link Mapping}s either from the
 * source or to the target {@link ONNode}.
 * 
 * @see DeleteMappingCommand
 * @see CreateMappingCommand  
 */

public class UpdateMappingCommand extends CompoundCommand {

	
	public static final String LABEL = "UpdateMappingCommand";

	Mapping mapping;

	Codomain codomain;

	ONNode source;

	ONNode target;
	
	public void update() {
		getCommands().clear();
		boolean createNewMapping = true;
		if (codomain == null || mapping == null ) return;
		for (Object mappingObject : codomain.getMappings()) {
			Mapping map = (Mapping) mappingObject;
			if (map.getTarget() == target || map.getSource() == source) {
				// if source AND target are already set, only delete the
				// connection!
				createNewMapping = (!((map.getSource() == source) && (map.getTarget() == target))) && createNewMapping;
				DeleteMappingCommand deleteMappingCommand = new DeleteMappingCommand();
				deleteMappingCommand.setMapping(map);
				add(deleteMappingCommand);
			}
		}
		if (createNewMapping) {
			CreateMappingCommand createMappingCommand = new CreateMappingCommand(mapping, codomain, source, target);
			add(createMappingCommand);
		}
	}

	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	public void setCodomain(Codomain codomain) {
		this.codomain = codomain;
	}

	public void setSource(ONNode source) {
		this.source = source;
	}

	public void setTarget(ONNode target) {
		this.target = target;
	}
	
	
}
