package roneditor.gef.editparts.objectnet;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.objectnet.ArcSelectionHandlesEditPolicy;
import roneditor.gef.editpolicies.objectnet.ONArcDirectEditPolicy;
import roneditor.gef.editpolicies.objectnet.ONArcEditPolicy;
import roneditor.gef.editpolicies.objectnet.ONArcEndpointEditPolicy;
import roneditor.gef.figures.objectnet.ArcFigure;
import roneditor.gef.tools.objectnet.ONArcCellEditorLocator;
import roneditor.gef.tools.objectnet.ONArcEditManager;
import roneditor.ui.views.properties.ONArcPropertySource;
import ronmodel.ONArc;
import ronmodel.RonmodelPackage;

/**
 * {@link EditPart} for a {@link ONArc}.
 */
public class ONArcEditPart extends AbstractConnectionEditPart {
	private IPropertySource propertySource = null;

	private Notifier target;

	private LinkAdapter adapter;

	/**
	 * Constructor
	 * 
	 * @param link
	 */
	public ONArcEditPart(ONArc link) {
		super();
		setModel(link);
	}

	/**
	 * Perform a DirectEdit (single click) via a request.
	 */
	protected void performDirectEdit() {
		new ONArcEditManager(this, new ONArcCellEditorLocator((ArcFigure) getFigure())).show();
	}

	/**
	 * Perform an Open request (double click). Currently calls
	 * performDirectEdit()
	 */
	private void performOpen() {
		performDirectEdit();
	}

	/**
	 * Handles request such as clicks on the EditPart.
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) // single
			// click
			performDirectEdit();
		else if (request.getType() == RequestConstants.REQ_OPEN) // double
			// click
			performOpen();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new ONArcEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ONArcEditPolicy());
		// install the edit policy for direct edititing of the label
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new ONArcDirectEditPolicy());

	}

	/**
	 * Get the current model casted as ONArc
	 * 
	 * @return the current model as ONArc.
	 */
	public ONArc getCastedModel() {
		return (ONArc) getModel();
	}

	/**
	 * Send a notification to the ONTransition that is connected to this ONArc.
	 */
	public void notifyTransition() {
		Notification notification = new NotificationImpl(0, true, false);
		ONArc arc = getCastedModel();
		arc.getTarget().eNotify(notification);
	}

	@Override
	protected IFigure createFigure() {
		return new ArcFigure(getCastedModel().getName(), getCastedModel().getWeight());
	}

	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new LinkAdapter();
		getCastedModel().eAdapters().add(adapter);
		super.activate();
	}

	@Override
	protected void refreshVisuals() {
		Connection c = getConnectionFigure();
		((ArcFigure) c).setWeight(getCastedModel().getWeight());
		((ArcFigure) c).setName(getCastedModel().getName());
		super.refreshVisuals();
	}

	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new ONArcPropertySource((ONArc) getModel());
		}
		return propertySource;
	}

	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class LinkAdapter implements Adapter {

		/**
		 * Handle notifications like changes of connections. Defaults to
		 * refreshVisuals().
		 * 
		 * @param notification
		 *            the {@link Notification} to process
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.ON_ARC__WEIGHT:
			case RonmodelPackage.ON_ARC__NAME:
				refreshVisuals();
				notifyTransition();
				break;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(type.getClass());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class

}
