package roneditor.gef.editparts.objectnet;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.objectnet.ONPlaceEditPolicy;
import roneditor.gef.figures.objectnet.PlaceFigure;
import roneditor.gef.tools.objectnet.ONPlaceCellEditorLocator;
import roneditor.gef.tools.objectnet.ONTransitionCellEditorLocator;
import roneditor.gef.tools.ron.NodeEditManager;
import roneditor.ui.views.properties.ONPlacePropertySource;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a ONPlaces. TODO: Write more stuff here!
 */
public class ONPlaceEditPart extends ONNodeEditPart implements NodeEditPart, Adapter {

	/**
	 * Constructor
	 * 
	 * @param place
	 */
	public ONPlaceEditPart(ONPlace place) {
		super(place);
	}

	/**
	 * Update the figure and refresh the display.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		((PlaceFigure) getNodeFigure()).setTokens(((ONPlace) getModel()).getTokens());
		super.refreshVisuals();
	}

	/**
	 * For convenience a double-click currently performs an DirectEdit.
	 */
	protected void performOpen() {
		performDirectEdit();
	}

	/**
	 * Get the current model casted as ONPlace
	 * 
	 * @return the current model as ONPlace.
	 */
	public ONPlace getCastedModel() {
		return (ONPlace) getModel();
	}

	/**
	 * Handle notifications like changes of connections. Defaults to
	 * refreshVisuals().
	 * 
	 * @param notification
	 *            the {@link Notification} to process
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.ON_PLACE__OUTGOING:
			refreshTargetConnections();
			break;
		case RonmodelPackage.ON_PLACE__INCOMING:
			refreshSourceConnections();
			break;
		default:
			refreshVisuals();
			break;
		}
		if (notification.getEventType() == Notification.SET)
			notifyTransitions();
	}

	/**
	 * Send a notification to all ONTransitions that are connected to this
	 * ONPlace via an ONArc.
	 */
	public void notifyTransitions() {
		Notification notification = new NotificationImpl(0, true, false);
		ONPlace place = getCastedModel();
		for (Object incomingArcObject : place.getIncoming()) {
			ONArc incomingArc = (ONArc) incomingArcObject;
			incomingArc.getSource().eNotify(notification);
		}

		for (Object outgoingArcObject : place.getOutgoing()) {
			ONArc outgoingArc = (ONArc) outgoingArcObject;
			outgoingArc.getTarget().eNotify(notification);
		}
	}

	/**
	 * Create the PlaceFigure as figure for the EditPart
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		PlaceFigure figure = new PlaceFigure();
		return figure;
	}

	/**
	 * Install the {@link ONPlace} specific edit policies (currently only the
	 * {@link ONPlaceEditPolicy}).
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new ONPlaceEditPolicy());
	}

	/**
	 * Perform the direct edit on the EditPart. Shows an {@link NodeEditManager}
	 * whose location is controlled by the {@link ONTransitionCellEditorLocator}.
	 */
	protected void performDirectEdit() {
		new NodeEditManager(this, new ONPlaceCellEditorLocator((PlaceFigure) getFigure())).show();
	}

	/**
	 * Return the appropiate {@link IPropertySource}, namely
	 * {@link ONPlacePropertySource} for the PropertyView
	 * 
	 * return The {@link IPropertySource} for the current Model
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new ONPlacePropertySource((ONPlace) getModel());
		}
		return propertySource;
	}

}
