package roneditor.gef.editparts.objectnet;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.gef.editpolicies.objectnet.ONTransitionEditPolicy;
import roneditor.gef.figures.objectnet.TransitionFigure;
import roneditor.gef.tools.objectnet.ONTransitionCellEditorLocator;
import roneditor.gef.tools.ron.NodeEditManager;
import roneditor.model.ONTransitionUtil;
import roneditor.ui.views.properties.ONTransitionPropertySource;
import ronmodel.ONTransition;
import ronmodel.RonmodelPackage;

/**
 * EditPart for an ONTransition. TODO: Write more stuff here!
 */
public class ONTransitionEditPart extends ONNodeEditPart implements NodeEditPart, Adapter {

	public ONTransitionEditPart(ONTransition server) {
		super(server);
	}

	/**
	 * This method is executed on an double click. Usability wise we normally
	 * fall back to direct edit but the transition should fire on a
	 * double-click.
	 * 
	 */
	protected void performOpen() {
		executeCommand(new FireTransitionCommand(getCastedModel()));
	}

	/**
	 * Create the edit policies for the Transition
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new ONTransitionEditPolicy());
	}

	/**
	 * Update the color of the figure depending if the transition can fire. Then
	 * execute super method.
	 * 
	 * @see objectnet.ONNodeEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		((TransitionFigure) getFigure()).setFireColor(canFire());
		super.refreshVisuals();
	}

	/**
	 * Handle notifications like changes of connections. Defaults to
	 * refreshVisuals().
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.ON_NODE__OUTGOING:
			refreshTargetConnections();
			break;
		case RonmodelPackage.ON_NODE__INCOMING:
			refreshSourceConnections();
			break;
		default:
			refreshVisuals();
			break;
		}
	}

	/**
	 * Return the current model casted as ONTranstition.
	 * 
	 * @return the current model as ONTransition
	 */
	public ONTransition getCastedModel() {
		return (ONTransition) getModel();
	}

	/**
	 * Check if the Transition of the model can fire.
	 * 
	 * @see ONTransitionUtil#transitionCanFire(ONTransition)
	 * 
	 * @return true if the Transition can fire
	 */
	public boolean canFire() {
		return ONTransitionUtil.transitionCanFire(getCastedModel());
	}

	/**
	 * Create the TransitionFigure as figure for the EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		TransitionFigure figure = new TransitionFigure();
		return figure;
	}

	/**
	 * Perform the direct edit on the EditPart. Shows an
	 * {@link NodeEditManager} whose location is controlled by the
	 * {@link ONTransitionCellEditorLocator}.
	 */
	protected void performDirectEdit() {
		new NodeEditManager(this, new ONTransitionCellEditorLocator((TransitionFigure) getFigure())).show();
	}

	/**
	 * Return the appropriate {@link IPropertySource}, namely
	 * {@link ONTransitionPropertySource} for the PropertyView
	 * 
	 * return The {@link IPropertySource} for the current Model
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new ONTransitionPropertySource((ONTransition) getModel());
		}
		return propertySource;
	}

}
