package roneditor.gef.editparts.objectnet;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.swt.widgets.Shell;

import roneditor.ui.part.RulePage;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Factory class for graphical edit parts. Here, the corresponding edit parts of
 * a network are created.
 */
public class ObjectNetEditPartFactory implements EditPartFactory {
	RulePage rulePage = null;

	public ObjectNetEditPartFactory() {
		super();
	}

	public ObjectNetEditPartFactory(RulePage rulePage) {
		super();
		this.rulePage = rulePage;
	}

	public ObjectNetEditPartFactory(Shell shell) {
		super();
	}

	/**
	 * Create an instance of the requested EditPart in the given context.
	 * Register the eAdapters of the Model in the rulePage if applicable.
	 * 
	 * @param context
	 *            The context
	 * @param obj
	 *            The type from the model to create the EditPart for
	 * 
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public EditPart createEditPart(EditPart context, Object obj) {
		if (obj instanceof ObjectNet) {
			ObjectNetEditPart p = new ObjectNetEditPart((ObjectNet) obj);
			return p;
		} else if (obj instanceof ONTransition) {
			ONTransitionEditPart p = new ONTransitionEditPart((ONTransition) obj);
			if (rulePage != null)
				((ONNode) obj).eAdapters().add(rulePage);
			return p;
		} else if (obj instanceof ONPlace) {
			ONPlaceEditPart p = new ONPlaceEditPart((ONPlace) obj);
			if (rulePage != null)
				((ONNode) obj).eAdapters().add(rulePage);
			return p;
		} else if (obj instanceof ONArc) {
			ONArcEditPart p = new ONArcEditPart((ONArc) obj);
			if (rulePage != null)
				((ONArc) obj).eAdapters().add(rulePage);
			return p;
		}
		return null;
	}
}
