package roneditor.gef.editparts.ron;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.objectnet.ArcSelectionHandlesEditPolicy;
import roneditor.gef.editpolicies.ron.HLArcEditPolicy;
import roneditor.gef.editpolicies.ron.HLArcEndpointEditPolicy;
import roneditor.gef.figures.ron.HLArcFigure;
import roneditor.ui.views.properties.HLArcPropertySource;
import ronmodel.HLArc;
import ronmodel.RonmodelPackage;

/**
 * {@link EditPart} for a {@link HLArc}.
 */
public class HLArcEditPart extends AbstractConnectionEditPart {
	private IPropertySource propertySource = null;

	private Notifier target;

	private LinkAdapter adapter;

	/**
	 * Constructor
	 * 
	 * @param link
	 */
	public HLArcEditPart(HLArc link) {
		super();
		setModel(link);
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new HLArcEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new HLArcEditPolicy());


	}

	/**
	 * Get the current model casted as ONArc
	 * 
	 * @return the current model as ONArc.
	 */
	public HLArc getCastedModel() {
		return (HLArc) getModel();
	}

	/**
	 * Send a notification to the ONTransition that is connected to this ONArc.
	 */
	public void notifyTransition() {
		Notification notification = new NotificationImpl(0, true, false);
		HLArc arc = getCastedModel();
		arc.getTarget().eNotify(notification);
	}

	@Override
	protected IFigure createFigure() {
		/*
		 * Connection c = (Connection) super.createFigure();
		 * ConnectionEndpointLocator relationshipLocator = new
		 * ConnectionEndpointLocator(c, false);
		 * relationshipLocator.setUDistance(5);
		 * relationshipLocator.setVDistance(10); Label relationshipLabel = new
		 * Label(getONArc().getName()); c.add(relationshipLabel,
		 * relationshipLocator);
		 */
		//TODO: name setzen
		return new HLArcFigure(getCastedModel().getName());
	}

	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new LinkAdapter();
		getCastedModel().eAdapters().add(adapter);
		super.activate();
	}

	@Override
	protected void refreshVisuals() {
		Connection c = getConnectionFigure();
		((HLArcFigure) c).setName(getCastedModel().getName());
		super.refreshVisuals();
	}

	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLArcPropertySource((HLArc) getModel());
		}
		return propertySource;
	}

	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class LinkAdapter implements Adapter {

		/**
		 * Handle notifications like changes of connections. Defaults to
		 * refreshVisuals().
		 * 
		 * @param notification
		 *            the {@link Notification} to process
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.HL_ARC__NAME:
				refreshVisuals();
				notifyTransition();
				break;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(type.getClass());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class

}
