package roneditor.gef.editparts.ron;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.objectnet.ONPlaceEditPolicy;
import roneditor.gef.editpolicies.ron.HLPlaceEditPolicy;
import roneditor.gef.editpolicies.ron.HLPlaceFlowLayoutEditPolicy;
import roneditor.gef.figures.ron.HLPlaceFigure;
import roneditor.gef.tools.objectnet.ONTransitionCellEditorLocator;
import roneditor.gef.tools.ron.HLPlaceCellEditorLocator;
import roneditor.gef.tools.ron.NodeEditManager;
import roneditor.ui.views.properties.HLPlacePropertySource;
import roneditor.ui.views.properties.ONPlacePropertySource;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.ONPlace;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a HLPlaces. TODO: Write more stuff here!
 */
public class HLPlaceEditPart extends HLNodeEditPart implements NodeEditPart, Adapter {

	/**
	 * Constructor
	 * 
	 * @param place
	 */
	public HLPlaceEditPart(HLPlace place) {
		super(place);
	}

	/**
	 * Update the figure and refresh the display.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		// Ensure type is correct
		((HLPlaceFigure) getFigure()).setType(getCastedModel().getType());
		// Refresh all children
		for (Object o : getChildren()) {
			((SimpleTokenEditPart) o).refreshVisuals();
		}
		super.refreshVisuals();
	}

	/**
	 * For convenience a double-click currently performs an DirectEdit.
	 */
	protected void performOpen() {
		// performDirectEdit();
	}

	/**
	 * Get the current model casted as ONPlace
	 * 
	 * @return the current model as ONPlace.
	 */
	public HLPlace getCastedModel() {
		return (HLPlace) getModel();
	}

	/**
	 * Handle notifications like changes of connections. Defaults to
	 * refreshVisuals().
	 * 
	 * @param notification
	 *            the {@link Notification} to process
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_PLACE__OUTGOING:
			refreshTargetConnections();
			refreshVisuals();
			notifyTransitions();
			break;
		case RonmodelPackage.HL_PLACE__INCOMING:
			refreshSourceConnections();
			notifyTransitions();
			break;

		case RonmodelPackage.HL_PLACE__TOKENS:
			refreshChildren();
			refreshVisuals();
			notifyTransitions();
			break;

		default:
			refreshVisuals();
			break;
		}
		if (notification.getEventType() == Notification.SET)
			notifyTransitions();
	}

	/**
	 * Send a notification to all HLTransitions that are connected to this
	 * ONPlace via an HLArc.
	 */
	public void notifyTransitions() {
		Notification notification = new NotificationImpl(0, true, false);
		HLPlace place = getCastedModel();
		for (Object incomingArcObject : place.getIncoming()) {
			HLArc incomingArc = (HLArc) incomingArcObject;
			if (incomingArc.getTarget() == null) continue;
			incomingArc.getSource().eNotify(notification);
		}

		for (Object outgoingArcObject : place.getOutgoing()) {
			HLArc outgoingArc = (HLArc) outgoingArcObject;
			if (outgoingArc.getTarget() == null) continue;
			outgoingArc.getTarget().eNotify(notification);
		}
	}

	/**
	 * Create the {@link HLPlaceFigure} as figure for the {@link EditPart}
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		HLPlaceFigure figure = new HLPlaceFigure();
		figure.setType(getCastedModel().getType());
		return figure;
	}

	/**
	 * Delegate the content pane to the content pane of the
	 * {@link HLPlaceFigure}
	 */
	@Override
	public IFigure getContentPane() {
		HLPlaceFigure placeFigure = (HLPlaceFigure) getFigure();
		return placeFigure.getContentPane();
	}

	/**
	 * Return the tokens on the {@link HLPlace} as it's children so the
	 * {@link EditPart} can be associated.
	 */
	@Override
	protected List getModelChildren() {
		return getCastedModel().getTokens();
	}

	/**
	 * Install the {@link ONPlace} specific edit policies (currently only the
	 * {@link ONPlaceEditPolicy}).
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new HLPlaceEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new HLPlaceFlowLayoutEditPolicy());
	}

	/**
	 * Perform the direct edit on the EditPart. Shows an {@link NodeEditManager}
	 * whose location is controlled by the {@link ONTransitionCellEditorLocator}.
	 */
	protected void performDirectEdit() {
		new NodeEditManager(this, new HLPlaceCellEditorLocator((HLPlaceFigure) getFigure())).show();
	}

	/**
	 * Return the appropriate {@link IPropertySource}, namely
	 * {@link ONPlacePropertySource} for the PropertyView
	 * 
	 * return The {@link IPropertySource} for the current Model
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLPlacePropertySource((HLPlace) getModel());
		}
		return propertySource;
	}

}
