package roneditor.gef.editparts.ron;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.ron.RONEditPolicy;
import roneditor.gef.editpolicies.ron.RONXYLayoutEditPolicy;
import roneditor.ui.views.properties.RONPropertySource;
import ronmodel.RON;
import ronmodel.RonmodelPackage;

/**
 * EditPart for the main component {@link RON}
 */
public class RONEditPart extends AbstractGraphicalEditPart implements Adapter, EditPartListener , LayerConstants {
	private IPropertySource propertySource = null;

	private Notifier target;

	private Shell shell;

	private Label nameLabel;

	/**
	 * Constructor
	 * 
	 * @param ron
	 *            the {@link RON}
	 */
	public RONEditPart(RON ron) {
		setModel(ron);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());
		layer.setOpaque(true);
		
		/**
		 * @author seb
		 * set a fancy ShortestPathConnectionRouter with FanRouter
		 */
		ConnectionLayer cLayer = (ConnectionLayer) getLayer(CONNECTION_LAYER);
		cLayer.setAntialias(SWT.ON);
		FanRouter frouter = new FanRouter();
		ShortestPathConnectionRouter router = new ShortestPathConnectionRouter(layer); 
		frouter.setSeparation(20);
		router.setSpacing(20);
		frouter.setNextRouter(router);
		cLayer.setConnectionRouter(frouter);
		
		return layer;
	}

	/**
	 * Install the EditPolicies for the EditPart. In particular we want to have
	 * the {@link RONEditPartGraphLayoutEditPolicy} assigned.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new RONEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new RONXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());
	//	installEditPolicy(ObjectNetEditPartGraphLayoutEditPolicy.ID, new ObjectNetEditPartGraphLayoutEditPolicy(this));
	}

	
	/**
	 * Returns the network.
	 * 
	 * @return the network
	 */
	public RON getCastedModel() {
		return (RON) getModel();
	}

	/**
	 * Return all children of the Model
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@SuppressWarnings("unchecked")
	protected List getModelChildren() {
		List children = new Vector();
		children.addAll(getCastedModel().getNodes());
		return children;
	}

	/**
	 * When activated the listener for changes on the model is registered and a
	 * {@link Label} showing the name of the {@link RON} is shown in the
	 * upper left corner of the {@link EditPart}
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// Listen for changes of the selection state
		addEditPartListener( this );

		// start listening for changes in the model
		((Notifier) getCastedModel()).eAdapters().add(this);
		
		updateFigure();

		nameLabel = new Label();
		nameLabel.setFont(JFaceResources.getFont("Sans14Bold"));
		nameLabel.setForegroundColor(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));
		nameLabel.setText(getCastedModel().getName());
		getFigure().add(nameLabel, new Rectangle(10, 10, -1, -1));
		super.activate();
	}
	
	
	
	/**
	 * Update the figure. 
	 * If the EditPart is selected, the border will be thickened.
	 */
	protected void updateFigure() {
		FreeformLayer layer = (FreeformLayer)getFigure();
		getFigure().setBackgroundColor(ColorConstants.white);
		if (getSelected() == 0) { // not selected
			layer.setBorder(new LineBorder(1));
			layer.setForegroundColor(new Color(null, 0,0,0));
		}
		else { // selected
			layer.setBorder(new LineBorder(2));
			layer.setForegroundColor(new Color(null, 150,0,0));
		}
	}
	

	

	@Override
	protected void refreshVisuals() {
		updateFigure();
		getFigure().repaint();
		super.refreshVisuals();
	}

	/**
	 * On deactivation the listener for model changes is removed.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;
		
		removeEditPartListener( this );
		
		// stop listening for changes in the model
		((Notifier) getCastedModel()).eAdapters().remove(this);

		super.deactivate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/*
		 * override the default behaviour defined in AbstractEditPart which would
		 * expect the model to be a property sourced. instead the EditPart can
		 * provide a property source
		 */
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new RONPropertySource(getCastedModel());
		}
		return propertySource;

	}

	/**
	 * Notify changes from the model to the view.
	 */
	public void notifyChanged(Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		if (featureId == RonmodelPackage.RON__NAME)
			nameLabel.setText(getCastedModel().getName());

		int type = notification.getEventType();
		switch (type) {
		case Notification.ADD:
		case Notification.ADD_MANY:
		case Notification.REMOVE:
		case Notification.REMOVE_MANY:
			refreshChildren();
			break;
		case Notification.SET:
			refreshVisuals();
			break;
		}
	}

	/**
	 * @return Returns the shell.
	 */
	public Shell getShell() {
		return shell;
	}

	/**
	 * @param shell
	 *            The shell to set.
	 */
	public void setShell(Shell shell) {
		this.shell = shell;
	}

	public void childAdded(EditPart child, int index) {}

	public void partActivated(EditPart editpart) {}

	public void partDeactivated(EditPart editpart) {}

	public void removingChild(EditPart child, int index) {}

	/**
	 * Called when the state of the selection is changed.
	 * Simply refreshes the visuals to show if the EditPart is selected.
	 */
	public void selectedStateChanged(EditPart editpart) {
		refreshVisuals();
	}
}
