package roneditor.gef.editparts.ron;

import java.util.Iterator;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.IRonEditorConstants;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.gef.editpolicies.ron.TokenComponentEditPolicy;
import roneditor.model.IDUtil;
import roneditor.ui.part.AbstractTreeEditor;
import roneditor.ui.part.RONTreeEditor;
import roneditor.ui.part.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.ui.utils.RONIconUtil;
import roneditor.ui.views.properties.ObjectNetPropertySource;
import roneditor.ui.views.properties.RulePropertySource;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import ronmodel.Token;

public class SimpleTokenEditPart extends AbstractGraphicalEditPart implements Adapter {

	private IPropertySource propertySource;

	public SimpleTokenEditPart(Token token) {
		setModel(token);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		super.activate();
		getCastedModel().eAdapters().add(this);
		// Listen to all children
		
		for (Iterator i = getCastedModel().eAllContents(); i.hasNext();) {
			EObject eo = (EObject) i.next();
			eo.eAdapters().add(this);
		}
		
	}

	@Override
	public void deactivate() {
		super.deactivate();
		getCastedModel().eAdapters().remove(this);
		/*
		// Remove Listener from all children
		for (Iterator i = getCastedModel().eAllContents(); i.hasNext();) {
			EObject eo = (EObject) i.next();
			eo.eAdapters().remove(this);
		}
		*/
	}

	/**
	 * Create a {@link Thumbnail} of the given {@link ObjectNet} of the given
	 * preferred {@link Dimension}.
	 * 
	 * @param net
	 * @param dimension
	 * @return The {@link Figure} of the {@link ObjectNet}
	 */
	public Figure getThumbnail(ObjectNet net, Dimension dimension) {
		// TODO: This part can probably be optimized... but works fine so far
		ObjectNetEditPartFactory factory = new ObjectNetEditPartFactory();
		ScrollingGraphicalViewer viewer = new ScrollingGraphicalViewer();
		Shell dialogShell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		viewer.createControl(new Composite(dialogShell, SWT.NONE));
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(factory);
		viewer.setContents(net);

		Thumbnail thumbnail = new Thumbnail();
		thumbnail.setSource(rootEditPart.getLayer(LayerConstants.PRINTABLE_LAYERS));
		thumbnail.setVisible(true);
		thumbnail.setSize(dimension);
		return thumbnail;
	}

	/**
	 * Create a {@link Thumbnail} of the {@link Token} that can be used as a
	 * tool tip.
	 * 
	 * @return
	 */
	public Figure getThumbnail() {
		if (getCastedModel() instanceof Rule) {
			Rule rule = ((Rule) getCastedModel());

			// The dimension for a single net in the rule
			Dimension dimension = new Dimension(100, 300);

			Figure thumbnail = new Figure();

			// Add the first NAC if it exists
			if (!rule.getNacs().isEmpty()) {
				Figure nac = getThumbnail((ObjectNet) rule.getNacs().get(0), dimension);
				thumbnail.add(nac);
			}

			// Add the LHS if there are any nodes on it
			if (rule.getLhs().getNodes().size() > 0) {
				Figure lhs = getThumbnail(rule.getLhs(), dimension);
				thumbnail.add(lhs);
			}

			// Add the RHS if there are any nodes on it
			if (rule.getRhs().getNodes().size() > 0) {
				Figure rhs = getThumbnail(rule.getRhs(), dimension);
				thumbnail.add(rhs);
				//System.out.println("SimpleTokenEditPart.getThumbnail() " + rhs.getSize());
			}

			// Use horizontal FlowLayout
			thumbnail.setLayoutManager(new FlowLayout(true));

			thumbnail.setBorder(new MarginBorder(3));

			return thumbnail;

		} else {

			// The dimension for the object net
			Dimension dimension = new Dimension(300, 300);

			Figure thumbnail = getThumbnail((ObjectNet) getModel(), dimension);

			thumbnail.setBorder(new MarginBorder(3));

			return thumbnail;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(final Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @return A ObjectNetPropertySource for the Properties View.
	 */
	private IPropertySource getPropertySource() {
		if (propertySource == null) {
			if (getCastedModel() instanceof Rule) {
				propertySource = new RulePropertySource((Rule) getCastedModel());
			} else {
				propertySource = new ObjectNetPropertySource((ObjectNet) getCastedModel());
			}
		}
		return propertySource;
	}

	@Override
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_OPEN) {
			if (getCastedModel() instanceof Rule) {
				openRuleView();
			} else {
				openObjectNetView();
			}
		}

	}

	/**
	 * Open the ObjectNetView of the current {@link Token}.
	 */
	private void openObjectNetView() {
		final IWorkbenchPage activePage = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		IEditorPart part = ((MultiViewerPageViewer) getViewer()).getHostPage().getEditor();
		AbstractTreeEditor treeEdit = (AbstractTreeEditor) part;
		try {
			activePage.showView(IRonEditorConstants.objectNetViewID, IDUtil.createIDForModel((ObjectNet) getCastedModel()),
					IWorkbenchPage.VIEW_ACTIVATE);
		} catch (final PartInitException e) {
			final String message = "View with ID " + IRonEditorConstants.objectNetViewID
					+ " could not be initialied in ObjectNetTreeEditPart.";
			((RONTreeEditor) ((DefaultEditDomain) getViewer().getEditDomain()).getEditorPart()).logError(message, e);
		}
	}

	/**
	 * Open the RuleView of the current {@link Token}.
	 */
	private void openRuleView() {
		final IWorkbenchPage activePage = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		IEditorPart part = ((MultiViewerPageViewer) getViewer()).getHostPage().getEditor();
		AbstractTreeEditor treeEdit = (AbstractTreeEditor) part;
		try {
			activePage.showView(IRonEditorConstants.ruleViewID, IDUtil.createIDForModel((Rule) getCastedModel()),
					IWorkbenchPage.VIEW_ACTIVATE);
		} catch (final PartInitException e) {
			final String message = "View with ID " + IRonEditorConstants.ruleViewID
					+ " could not be initialied in RuleTreeEditPart.";
			((RONTreeEditor) ((DefaultEditDomain) getViewer().getEditDomain()).getEditorPart()).logError(message, e);
		}
	}

	/**
	 * Create an Label as a Figure with it's type specific icon depending on the
	 * {@link HLPlaceType}.
	 */
	@Override
	protected IFigure createFigure() {
		Label label = new Label();
		label.setText(getCastedModel().getName());
		label.setIcon(RONIconUtil.getInstance().getIcon(RonmodelPackage.TOKEN, getCastedModel().getPlace().getType().getValue(),
				16));

		return label;
	}

	public Token getCastedModel() {
		return (Token) getModel();
	}

	public Label getLabel() {
		return (Label) getFigure();
	}

	@Override
	protected void refreshVisuals() {
		// high-light the token if it is first in the parent place
		HLPlace place = ((HLPlaceEditPart) getParent()).getCastedModel();
		boolean isFirst = false;
		if (place.getTokens().size() > 0) {
			Token firstChild = (Token) place.getTokens().get(0);
			isFirst = (firstChild == getCastedModel());
		}

		if (isFirst)
			getLabel().setForegroundColor(ColorConstants.black);
		else
			getLabel().setForegroundColor(ColorConstants.gray);

		getLabel().setText(getCastedModel().getName());
		Rectangle bounds = new Rectangle(new Point(0, 0), getFigure().getPreferredSize());
		((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), bounds);

		/**
		 * RectangleFigure e = new RectangleFigure(); e.setSize(200, 200);
		 * getLabel().setToolTip(e);
		 */
		getLabel().setToolTip(getThumbnail());

		super.refreshVisuals();
	}

	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new TokenComponentEditPolicy());
	}

	public Notifier getTarget() {
		return null;
	}

	public boolean isAdapterForType(Object type) {
		return false;
	}

	public void notifyChanged(Notification notification) {
		if ( notification.getEventType() == 0) return; // Ignore Wake-up notification
		refreshVisuals();
	}

	public void setTarget(Notifier newTarget) {
	}

}
