/**
 * 
 */
package roneditor.gef.editparts.rule;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import roneditor.gef.editpolicies.rule.MappingEditPolicy;
import ronmodel.Mapping;
import ronmodel.RonmodelPackage;

/**
 * {@link EditPart} for {@link Mapping}. 
 */
public class MappingEditPart extends AbstractConnectionEditPart {

	private Notifier target;

	private LinkAdapter adapter;

	/**
	 * Constructor
	 * 
	 * @param link
	 */
	public MappingEditPart(Mapping mapping) {
		super();
		setModel(mapping);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new LinkAdapter();
		getCastedModel().eAdapters().add(adapter);
		super.activate();
	}

	/**
	 * Return the active model as {@link Mapping}.
	 * 
	 * @return
	 */
	public Mapping getCastedModel() {
		return (Mapping) getModel();
	}

	/**
	 * Install the {@link MappingEditPolicy} on connection.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new MappingEditPolicy());
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class LinkAdapter implements Adapter {

		/**
		 * Handle notifications like changes of connections. Defaults to
		 * refreshVisuals().
		 * 
		 * @param notification
		 *            the {@link Notification} to process
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.MAPPING__SOURCE:
			case RonmodelPackage.MAPPING__TARGET:
				refreshVisuals();
				break;
			}
			
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			System.out.println("LinkAdapter.isAdapterForType() =>"+type+" =>"+type.getClass());
			return type.equals(type.getClass());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class

}
