package roneditor.gef.editpolicies.ron;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.editparts.ron.HLNodeEditPart;
import roneditor.gef.figures.objectnet.NodeFigure;
import ronmodel.Codomain;
import ronmodel.HLNode;
import ronmodel.Mapping;

/**
 * {@link GraphicalNodeEditPolicy} for the {@link HLNodeEditPart}.
 * Provides some convenience functions and initiates the feedback
 * and connection handling of {@link Mapping} commands. 
 */
public class HLNodeEditPolicy extends GraphicalNodeEditPolicy {

	public HLNodeEditPolicy() {
		super();
	}

	/**
	 * Return the casted model of the host {@link EditPart}.
	 * 
	 * @return
	 */
	protected HLNode getCastedModel() {
		return (HLNode) getHost().getModel();
	}

	/**
	 * Return the {@link EditPartViewer} of the host {@link EditPart}.
	 * 
	 * @return
	 */
	public EditPartViewer getViewer() {
		return getHost().getRoot().getViewer();
	}

	/**
	 * Return the {@link Codomain} of the {@link EditPartViewer}.
	 * 
	 * @return
	 */
	public Codomain getCodomain() {
		Object model = getViewer().getContents().getModel();
		if (model instanceof Codomain)
			return (Codomain) model;
		return null;
	}

	/**
	 * If we create a {@link Mapping} we want to have another feedback. This
	 * method is called while the connection is not finished.
	 */
	@Override
	public void showSourceFeedback(Request request) {
		if (request instanceof CreateConnectionRequest) {
			CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			Object requestingObject = createConnectionRequest.getNewObject();
			if (requestingObject instanceof Mapping) {
				((NodeFigure) getHostFigure()).setGlow(true);

				return;
			}
		}
		super.showSourceFeedback(request);
	}

	/**
	 * If we have create a {@link Mapping} we want to have another feedback.
	 * This method is called when the connection is finished.
	 */
	@Override
	public void eraseSourceFeedback(Request request) {
		if (request instanceof CreateConnectionRequest) {
			CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			Object requestingObject = createConnectionRequest.getNewObject();
			if (requestingObject instanceof Mapping) {
				((NodeFigure) getHostFigure()).setGlow(false);
				return;
			}
		}
		super.eraseSourceFeedback(request);
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the
	 * completion {@link Command}. When our start {@link Command} is an
	 */
	@Override
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		return null;
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the creation
	 * {@link Command}. Handles the creation of the
	 */
	@Override
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		return null;
	}

}
