package roneditor.gef.editpolicies.ron;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.FlowLayoutEditPolicy;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.objectnet.ChangeTokenOrderCommand;
import roneditor.gef.commands.ron.CreateTokenCommand;
import roneditor.gef.commands.ron.MoveMultipleTokenCommand;
import roneditor.gef.editparts.ron.HLPlaceEditPart;
import roneditor.gef.editparts.ron.SimpleTokenEditPart;
import roneditor.model.RONModelManager;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * EditPolicy for XY-Layout.
 */
public class HLPlaceFlowLayoutEditPolicy extends FlowLayoutEditPolicy {

	
	/**
	 * The {@link Request} should be of type {@link ChangeBoundsRequest} which
	 * contains the moved {@link EditPart}s. Hence we return a
	 * {@link MoveMultipleTokenCommand} that handles the actual model change.
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected Command createAddCommand(EditPart edit,EditPart after) {
		if (edit instanceof SimpleTokenEditPart){
			List modelList = new LinkedList();
			modelList.add(((EditPart) edit).getModel());
			return new MoveMultipleTokenCommand(modelList, getCastedModel());
		}
		return null;
	}
	
	/**
	 * @return The model of the host {@link EditPart} casted as {@link HLPlace}.
	 */
	public HLPlace getCastedModel() {
		return (HLPlace) getHost().getModel();
	}
	

	/**
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Object newObjectType = request.getNewObjectType();
		Command createCommand = null;
		
		if (newObjectType == Token.class) {
			CreateTokenCommand create;
			if (getCastedModel().getType().equals(HLPlaceType.NET_LITERAL)) {
				final ObjectNet newToken = RONModelManager.getFactory().createObjectNet();
				newToken.setName("neu");
				create = new CreateTokenCommand(getCastedModel(),newToken);
			} else {
				final Rule newToken = RONModelManager.getFactory().createRule();
				newToken.setLhs(RONModelManager.getFactory().createLHS());
				newToken.setRhs(RONModelManager.getFactory().createRHS());
				newToken.getLhs().setName("lhs");
				newToken.getRhs().setName("rhs");
				create = new CreateTokenCommand(getCastedModel(),newToken);
			}
			create.setLabel("Create a token");
			create.setName("neu");
			createCommand = create;
		}

		return createCommand;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		//if (child instanceof ONPlaceEditPart)
		//	return new PlaceFigureResizableEditPolicy();
		//if (child instanceof ONTransitionEditPart)
		//	return new TransitionFigureResizableEditPolicy();
		return new NonResizableEditPolicy();
	}


	@Override
	protected Command createMoveChildCommand(EditPart child, EditPart after) {
		return new ChangeTokenOrderCommand((HLPlaceEditPart)getHost(),(SimpleTokenEditPart)child, (SimpleTokenEditPart)after);
	}

}
