package roneditor.gef.editpolicies.ron;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.ron.MoveMultipleTokenCommand;
import roneditor.gef.editparts.tree.HLPlaceTreeEditPart;
import ronmodel.HLPlace;
import ronmodel.Token;

/**
 * {@link LayoutEditPolicy} for the {@link HLPlaceTreeEditPart}, used to
 * support drag and drop of {@link Token}.
 */
public class HLPlaceLayoutEditPolicy extends LayoutEditPolicy {

	/**
	 * The {@link Request} should be of type {@link ChangeBoundsRequest} which
	 * contains the moved {@link EditPart}s. Hence we return a
	 * {@link MoveMultipleTokenCommand} that handles the actual model change.
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected Command getAddCommand(Request request) {
		if (request instanceof ChangeBoundsRequest) {
			ChangeBoundsRequest cbr = (ChangeBoundsRequest) request;
			// Extract the Model Objects from the EditParts
			List modelList = new LinkedList();
			for (Object editPart : cbr.getEditParts()) {
				modelList.add(((EditPart) editPart).getModel());
			}
			return new MoveMultipleTokenCommand(modelList, getCastedModel());
		}
		return super.getAddCommand(request);
	}

	/**
	 * @return The model of the host {@link EditPart} casted as {@link HLPlace}.
	 */
	public HLPlace getCastedModel() {
		return (HLPlace) getHost().getModel();
	}

	@Override
	protected EditPolicy createChildEditPolicy(EditPart child) {
		return null;
	}

	@Override
	protected Command getCreateCommand(CreateRequest request) {
		return null;
	}

	@Override
	protected Command getMoveChildrenCommand(Request request) {
		return null;
	}

}
