package roneditor.gef.editpolicies.ron;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.ron.ConnectHLPlaceTransitionCommand;
import roneditor.gef.commands.ron.ConnectHLTransitionPlaceCommand;
import roneditor.gef.commands.ron.ReconnectHLNodeCommand;
import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ONTransition;

/**
 * {@link EditPolicy} for an {@link HLTransitionEditPart}.
 */
public class HLTransitionEditPolicy extends HLNodeEditPolicy {

	public static String FIRE_REQUEST = "HLTransitionEditPolicyFire";

	public HLTransitionEditPolicy() {
		super();
	}

	public HLTransition getCastedModel() {
		return (HLTransition) super.getCastedModel();
	}

	/**
	 * When ever a connection is completed, we update the provided {@link CreateConnectionRequest} with the
	 * target, hence the {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		if (command instanceof ConnectHLPlaceTransitionCommand) {
			ConnectHLPlaceTransitionCommand commandSC = (ConnectHLPlaceTransitionCommand) command;
			commandSC.setTarget(getCastedModel());
			return command;
		}
		return super.getConnectionCompleteCommand(request);
	}


	/**
	 * When ever a connection is created, we update the provided {@link CreateConnectionRequest} with the
	 * source, hence the {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */

	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		Object requestingObject = request.getNewObject();
		if (requestingObject instanceof HLArc) {
			ConnectHLTransitionPlaceCommand command = new ConnectHLTransitionPlaceCommand();
			request.setStartCommand(command);
			command.setArc((HLArc) requestingObject);
			command.setSource(getCastedModel());
			return command;
		}
		return super.getConnectionCreateCommand(request);
	}

	/**
	 * When ever a connection is changed, we update the provided {@link ReconnectRequest} with the
	 * target, hence the {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		ReconnectHLNodeCommand command = new ReconnectHLNodeCommand();
		HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		EditPart target = request.getTarget();
		if (link.getTarget() instanceof HLPlace && link.getSource() != target.getModel()) {
			command.setArc(link);
			command.setSource((HLNode) target.getModel());
			command.setTarget(link.getTarget());
			return command;
		} else
			return null;
	}

	/**
	 * When ever a connection is changed, we update the provided {@link ReconnectRequest} with the
	 * source, hence the {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		ReconnectHLNodeCommand command = new ReconnectHLNodeCommand();
		HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		EditPart target = request.getTarget();
		if (link.getSource() instanceof HLPlace && link.getTarget() != target.getModel()) {
			command.setArc(link);
			command.setSource(link.getSource());
			command.setTarget((HLNode) target.getModel());
			return command;
		} else
			return null;
	}

}
