package roneditor.gef.editpolicies.ron;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.ron.CreateHLNodeCommand;
import roneditor.gef.commands.ron.CreateHLPlaceCommand;
import roneditor.gef.commands.ron.CreateHLTransitionCommand;
import roneditor.gef.commands.ron.SetConstraintCommand;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.RON;

/**
 * EditPolicy for XY-Layout.
 */
public class RONXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/**
	 * Handles requests for creation of {@link Command} that generate an
	 * {@link HLPlace} or {@link HLTransition}.
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Object newObjectType = request.getNewObjectType();
		Command createCommand = null;

		if (newObjectType == HLTransition.class || newObjectType == HLPlace.class) {


			CreateHLNodeCommand create;
			if (newObjectType == HLTransition.class) {
				create = new CreateHLTransitionCommand();
				create.setLabel("Create a transition");
			} else {
				create = new CreateHLPlaceCommand();
				create.setLabel("Create a place");
			}
			create.setHLNode((HLNode) request.getNewObject());
			create.setRON((RON) getHost().getModel());
			
			// seb: begin - location workaround
			Point p = request.getLocation();
			this.getHost().getViewer();
			FigureCanvas canvas = (FigureCanvas) this.getHost().getViewer().getControl();
			Point delta = canvas.getViewport().getViewLocation();
			p.translate(delta);
			p.translate(-10, -10);
			// seb: end - location workaroun
			create.setX(p.x);
			create.setY(p.y);

			createCommand = create;
		}

		return createCommand;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		//if (child instanceof ONPlaceEditPart)
		//	return new PlaceFigureResizableEditPolicy();
		//if (child instanceof ONTransitionEditPart)
		//	return new TransitionFigureResizableEditPolicy();
		return new NonResizableEditPolicy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getMinimumSizeFor(org.eclipse.gef.GraphicalEditPart)
	 */
	protected Dimension getMinimumSizeFor(GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}

	/**
	 * @return A SetConstraintCommand with the model of th host EditPart and the
	 *         constraint of its new delta set.
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		// final MoveCommand locationCommand = new MoveCommand((ONNode) child
		// .getModel(), ((Rectangle) constraint).getLocation());
		// return locationCommand;
		SetConstraintCommand locationCommand = new SetConstraintCommand();
		locationCommand.setHLNode((HLNode) child.getModel());
		locationCommand.setNewPosition(((Rectangle) constraint).getLocation());
		return locationCommand;
	}

}
