package roneditor.gef.figures.objectnet;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

/**
 * Represents the graphical layout of a transition.
 */
public class PlaceFigure extends NodeFigure {
	protected Label tokenLabel;

	protected Ellipse placeEllipse;

	protected int tokens;

	protected int RADIUS = 30;

	protected Color defaultColor = new Color(null, 224, 229, 255);

	public PlaceFigure() {
		super();
		setOutline(false);
		setFill(false);
		tokens = 0;

		placeEllipse = new Orb();
		placeEllipse.setBackgroundColor(defaultColor);
		placeEllipse.setFill(true);
		add(placeEllipse);

		tokenLabel = new Label(" ");
		add(tokenLabel);

		incomingConnectionAnchor = new ChopboxAnchor(placeEllipse);
		outgoingConnectionAnchor = new ChopboxAnchor(placeEllipse);

		setRuleMode(false);
	}

	public void setGlow(boolean glow) {
		if (glow) {
			placeEllipse.setForegroundColor(ColorConstants.red);
		} else {
			placeEllipse.setForegroundColor(ColorConstants.black);
		}
		super.setGlow(glow);
	}

	public Rectangle getEllipseBounds() {
		return new Rectangle(getDiameter() / 2 - RADIUS / 2, 0, RADIUS, RADIUS);
	}

	public Rectangle getLabelBounds() {
		Dimension d = nameLabel.getPreferredSize();
		return new Rectangle((getDiameter() - d.width) / 2, RADIUS + 4, -1, -1);
	}

	public Rectangle getLabelExactBounds() {
		Dimension d = nameLabel.getPreferredSize();
		Rectangle r = getLabelBounds();
		r.width = d.width;
		r.height = d.height;
		return r;
	}

	public Rectangle getTokenLabelBounds() {
		Dimension d = tokenLabel.getPreferredSize();
		return new Rectangle((getDiameter() - d.width) / 2, 1 + (RADIUS - d.height) / 2, -1, -1);
	}

	public void setTokens(int tokens) {
		this.tokens = tokens;
		if (tokens > 0)
			tokenLabel.setText(String.valueOf(tokens));
		else
			tokenLabel.setText("");
		updateLayout();
	}

	public int getTokens() {
		return tokens;
	}

	public void makeGhostShape() {
		FigureUtilities.makeGhostShape(placeEllipse);
		placeEllipse.setLineStyle(Graphics.LINE_DOT);
		placeEllipse.setForegroundColor(ColorConstants.white);
		setFill(false);
		setLineWidth(0);
		setOutline(false);
	}

	protected int getDiameter() {
		return Math.max(nameLabel.getPreferredSize().width, RADIUS);
	}

	public void updateLayout() {
		layout.setConstraint(nameLabel, getLabelBounds());
		layout.setConstraint(placeEllipse, getEllipseBounds());
		layout.setConstraint(tokenLabel, getTokenLabelBounds());
	}

	public void validate() {
		updateLayout();
		super.validate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int diam = Math.max(nameLabel.getSize().width, 40);
		return new Dimension(diam, 50);
	}

	/**
	 * Sets the new size
	 * 
	 * @param w
	 *            the new width
	 * @param h
	 *            the new height
	 * 
	 */
	public void setSize(int w, int h) {
		height = h;
		width = w;
		RADIUS = calcRadius();
		updateLayout();
	}

	/**
	 * calculates the radius
	 * 
	 * @return the new radius
	 */
	private int calcRadius() {
		return Math.round((float) Math.sqrt((width * width) + (height * height)));
	}

	public void setColor(Color c) {
		placeEllipse.setBackgroundColor(c);
	}
	
	public Color getColor() {
		return placeEllipse.getBackgroundColor();
	}

}