package roneditor.gef.figures.objectnet;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
/**
 * Represents the graphical layout of a place.
 */
public class TransitionFigure extends NodeFigure {
	boolean fireColor;

	public TransitionFigure() {
		super();
		// setBackgroundColor(new Color(null, 225, 255, 220));
		incomingConnectionAnchor = new ChopboxAnchor(this);
		outgoingConnectionAnchor = new ChopboxAnchor(this);
		setFireColor(false);
		setRuleMode(false);
	}

	/**
	 * Fill the shape with a fancy gradient.
	 */
	@Override
	protected void fillShape(Graphics graphics) {
		if ( graphics.getXORMode() )
			graphics.fillRectangle(getBounds());
		else
			graphics.fillGradient(getBounds(),true);
	}
	
	/**
	 * Draw the outline of the shape. Red when glowing, black otherwise.
	 */
	@Override
	protected void outlineShape(Graphics graphics) {
		if ( glow ) {
			graphics.setForegroundColor(ColorConstants.red);
		}
		else {
			graphics.setForegroundColor(ColorConstants.black);
		}
		super.outlineShape(graphics);
	}

	/**
	 * @see Shape#outlineShape(Graphics)
	 */
	protected void glowShape(Graphics graphics) {
		Rectangle r = getBounds();
		for (int i=0; i<4; i++) {
			int x = r.x + i;
			int y = r.y + i;
			int w = r.width - i*2;
			int h = r.height - i*2;
			graphics.drawRectangle(x, y, w, h);
		}
	}
	
	public void setIcon(Image image) {
		nameLabel.setIcon(image);
	}

	public void validate() {
		layout.setConstraint(nameLabel, new Rectangle(5, 5, -1, -1));
		super.validate();
	}

	public void setFireColor(boolean fireColor) {
		this.fireColor = fireColor;
		if (!ruleMode) {
			if (fireColor) {
				setColor(new Color(null, 225, 255, 220));
			} else {
				setColor(new Color(null, 255, 170, 170));
			}
		}
	}
	
	public Color getColor() {
		return getForegroundColor();
	}

	public void setColor(Color c) {
		float hsv[] = c.getRGB().getHSB();
		float hue = hsv[0];
		float satuartion = hsv[1];
		float value = hsv[2];
		Color fg;
		if (hue > 180 /* && f[0] < 300 */) {
			fg = new Color(null, 255, 255, 255);
		} else
			fg = new Color(null, 0, 0, 0);
		nameLabel.setForegroundColor(fg);
		if ( highlight )
			value *= 0.9;
		else
			value *= 0.8;
			
		setForegroundColor(c);
		setBackgroundColor(new Color(null, new RGB(hue, satuartion, value)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int diam = Math.max(nameLabel.getSize().width, nameLabel.getSize().height) + 10;
		return new Dimension(diam, 22);
	}

}