package roneditor.gef.figures.ron;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import roneditor.gef.figures.objectnet.NodeFigure;
import roneditor.gef.figures.objectnet.Orb;
import roneditor.gef.figures.objectnet.ShadedRectangleFigure;
import ronmodel.HLPlaceType;

public class HLPlaceFigure extends NodeFigure {

	static protected final int NET_TYPE = 0;

	static protected final int RULE_TYPE = 1;

	static protected final int MINIMUM_WIDTH = 100;

	static protected final int RADIUS = 30;

	static protected final int MARGIN = 10;

	protected RectangleFigure contentRectangle;

	protected ShadedRectangleFigure backgroundRectangle;

	protected Orb orb;

	protected Label typeLabel;
	
	public HLPlaceFigure() {
		layout = new XYLayout();
		setLayoutManager(layout);

		setOutline(false);
		setFill(false);
		setOpaque(false);
		setBackgroundColor(ColorConstants.white);
		setForegroundColor(ColorConstants.black);

		backgroundRectangle = new ShadedRectangleFigure();
		backgroundRectangle.setOutline(true);
		backgroundRectangle.setOpaque(true);
		backgroundRectangle.setFill(true);
		backgroundRectangle.setForegroundColor(new Color(null, 255, 255, 255));
		backgroundRectangle.setForegroundColor(new Color(null, 200, 200, 200));
		add(backgroundRectangle);

		contentRectangle = new RectangleFigure();
		contentRectangle.setOutline(false);
		contentRectangle.setOpaque(false);
		contentRectangle.setFill(false);
		FlowLayout flowLayout = new FlowLayout();
		flowLayout.setHorizontal(false);
		flowLayout.setMinorSpacing(5);
		flowLayout.setMajorAlignment(FlowLayout.ALIGN_LEFTTOP);
		contentRectangle.setLayoutManager(flowLayout);
		add(contentRectangle);

		nameLabel = new Label(" ");
		nameLabel.setFont(JFaceResources.getFont("Sans12Bold"));
	    Font initialFont = nameLabel.getFont();
	    FontData[] fontData = initialFont.getFontData();
	    for (int i = 0; i < fontData.length; i++) {
	    	fontData[i].setStyle(SWT.BOLD);
	    }
	    Font newFont = new Font(null, fontData);
	    nameLabel.setFont(newFont);
		add(nameLabel);

		orb = new Orb();
		add(orb);
		
		typeLabel = new Label(" ");
		typeLabel.setFont(JFaceResources.getFont("Sans14Bold"));
		add(typeLabel);

		incomingConnectionAnchor = new ChopboxAnchor(this);
		outgoingConnectionAnchor = new ChopboxAnchor(this);

	}

	public void setType(HLPlaceType type) {
		Color color1, color2;
		if (type.equals(HLPlaceType.NET_LITERAL)) {
			typeLabel.setText("O");
			color1 = new Color(null, 183, 226, 250);
			color2 = new Color(null, 37, 148, 254);
		}
		else {
			typeLabel.setText("R");
			color1 = new Color(null, 198, 250, 183);
			color2 = new Color(null, 86, 254, 37);
		}
		backgroundRectangle.setForegroundColor(ColorConstants.gray);
		backgroundRectangle.setGradientColor(color1, color2);
		orb.setBackgroundColor(color1);
		
	}

	protected void updateLayout() {

		int w = Math.max(MINIMUM_WIDTH, nameLabel.getPreferredSize().width) ;
		w = Math.max(w, contentRectangle.getPreferredSize().width);
		w += 2 * MARGIN;
		
		layout.setConstraint(orb, new Rectangle((w - RADIUS) / 2, 0, RADIUS, RADIUS));
		layout.setConstraint(typeLabel, new Rectangle((w - RADIUS) / 2, 0, RADIUS, RADIUS));

		int y = nameLabel.getPreferredSize().height + 5;

		layout.setConstraint(nameLabel, new Rectangle(MARGIN, RADIUS, -1, -1));
		layout.setConstraint(contentRectangle, new Rectangle(MARGIN, RADIUS + y, w, -1));

		int h = contentRectangle.getPreferredSize().height + y + (RADIUS - MARGIN);
		layout.setConstraint(backgroundRectangle, new Rectangle(0, RADIUS - MARGIN, w, h));
	}

	public void validate() {
		updateLayout();
		super.validate();
	}

	public RectangleFigure getContentPane() {
		return contentRectangle;
	}

	public Rectangle getLabelClientArea() {
		return nameLabel.getClientArea();
	}
	
	
	public Dimension getPreferredSize(int wHint, int hHint) {
		updateLayout();
		int w = Math.max(MINIMUM_WIDTH, nameLabel.getPreferredSize().width) ;
		w = Math.max(w, contentRectangle.getPreferredSize().width);
		w += 2 * MARGIN;
		int h = 2 * (RADIUS - MARGIN) + contentRectangle.getPreferredSize().height + nameLabel.getPreferredSize().height + 5;
		return new Dimension(w, h);
	}

}
