package roneditor.gef.palette.objectnet;

import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import roneditor.model.ModelCreationFactory;
import roneditor.ui.utils.RONIconUtil;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelPackage;

/**
 * Main class for the network editor palette. All graphical figures have to be
 * registered here.
 */
public class ObjectNetPaletteRoot extends PaletteRoot {
	protected PaletteGroup symbols;
	
	
	/**
	 * Creates a new NetworkPaletteRoot instance.
	 * 
	 */
	public ObjectNetPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		PaletteGroup controls = new PaletteGroup("Controls");
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols
		symbols = new PaletteGroup("Symbols");
		add(symbols);

		CreationToolEntry entry;

		entry = new ObjectNetCreationToolEntry("Transition", "Creates a transition",
				new ModelCreationFactory(ONTransition.class), 
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_TRANSITION ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_TRANSITION ,24)
		);
		symbols.add(entry);

		entry = new ObjectNetCreationToolEntry("Place", "Creates a place", new ModelCreationFactory(ONPlace.class),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_PLACE,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_PLACE ,24)
		);
		symbols.add(entry);

		entry = new ObjectNetConnectionCreationToolEntry("Arc", "Creates a arc", new ModelCreationFactory(ONArc.class),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_ARC ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.ON_ARC ,24)
		);
		symbols.add(entry);

		entry = new ObjectNetMagicConnectionCreationToolEntry( "MagicArc", "Creates a arc in magic way", new ModelCreationFactory(ONArc.class), 
				RONIconUtil.getInstance().getDescriptor(RONIconUtil.MAGIC_ARC,16),
				RONIconUtil.getInstance().getDescriptor(RONIconUtil.MAGIC_ARC,24)
		);
		symbols.add(entry);
	}
}
