package roneditor.gef.palette.ron;

import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import roneditor.model.ModelCreationFactory;
import roneditor.ui.utils.RONIconUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.RonmodelPackage;
import ronmodel.Token;


/**
 * Main class for the ron editor palette. All graphical figures have to be
 * registered here.
 */
public class RONPaletteRoot extends PaletteRoot {

	/**
	 * Creates a new NetworkPaletteRoot instance.
	 * 
	 */
	public RONPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		PaletteGroup controls = new PaletteGroup("Controls");
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols
		PaletteGroup symbols = new PaletteGroup("Symbols");
		add(symbols);

		CreationToolEntry entry;

		
		
		entry = new HLTransitionCreationToolEntry("STANDARD", "Creates a STANDARD transition",new ModelCreationFactory(HLTransition.class),
				HLTransitionKind.STANDARD_LITERAL,
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.STANDARD ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.STANDARD ,24)
				);
		symbols.add(entry);
		entry = new HLTransitionCreationToolEntry("FIRE", "Creates a FIRE transition",new ModelCreationFactory(HLTransition.class),
				HLTransitionKind.FIRE_LITERAL,
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.FIRE ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.FIRE ,24)
				);
		symbols.add(entry);
		entry = new HLTransitionCreationToolEntry("APPLYRULE", "Creates a APPLYRULE transition",new ModelCreationFactory(HLTransition.class),
				HLTransitionKind.APPLYRULE_LITERAL,
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.APPLYRULE ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.APPLYRULE ,24)
		);
		symbols.add(entry);
		entry = new HLTransitionCreationToolEntry("SPLIT", "Creates a SPLIT transition",new ModelCreationFactory(HLTransition.class),
				HLTransitionKind.SPLIT_LITERAL,
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.SPLIT ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_TRANSITION, HLTransitionKind.SPLIT ,24)
		);
		symbols.add(entry);
		
		entry = new HLPlaceCreationToolEntry("ObjectNetPlace", "Creates a place for a objectNets", new ModelCreationFactory(HLPlace.class),
				HLPlaceType.NET_LITERAL,
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_PLACE, HLPlaceType.NET,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_PLACE, HLPlaceType.NET ,24)
		);
		symbols.add(entry);

		entry = new HLPlaceCreationToolEntry("RulePlace", "Creates a place for a rule", new ModelCreationFactory(HLPlace.class),
				HLPlaceType.RULE_LITERAL,

				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_PLACE, HLPlaceType.RULE,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_PLACE, HLPlaceType.RULE,24)
		);
		symbols.add(entry);

		entry = new TokenCreationToolEntry("Token", "Creates a token", new ModelCreationFactory(Token.class),
				RONIconUtil.getInstance().getDescriptor(RONIconUtil.TOKEN, 16),
				RONIconUtil.getInstance().getDescriptor(RONIconUtil.TOKEN, 24)
		);
		symbols.add(entry);	
	
		entry = new ConnectionCreationToolEntry("Arc", "Creates a arc", new ModelCreationFactory(HLArc.class),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_ARC ,16),
				RONIconUtil.getInstance().getDescriptor(RonmodelPackage.HL_ARC ,24)
		);
		symbols.add(entry);

	}
}
