package roneditor.gef.tools.objectnet;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

import roneditor.gef.editparts.objectnet.ONArcEditPart;
import roneditor.gef.figures.objectnet.ArcFigure;

/**
 * The {@link CellEditorLocator} for DirectEdit operations of the
 * {@link ONArcEditPart}. Determines the location of the DirectEdit
 * {@link CellEditor}.
 */
public class ONArcCellEditorLocator implements CellEditorLocator {
	private ArcFigure arcFigure;

	public ONArcCellEditorLocator(ArcFigure placeFigure) {
		setFigure(placeFigure);
	}

	/**
	 * Determine the position for the CellEditor based on the attached
	 * ArcFigure.
	 */
	public void relocate(CellEditor celleditor) {
		Text text = (Text) celleditor.getControl();
		Rectangle rect = arcFigure.getLabelArea();
		org.eclipse.swt.graphics.Point size = text.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		org.eclipse.swt.graphics.Rectangle trim = text.computeTrim(0, 0, 0, 0);
		rect.translate(trim.x, trim.y);
		rect.width = Math.max(size.x, rect.width) + 5;
		rect.width += trim.width;
		rect.height += trim.height;
		text.setBounds(rect.x, rect.y, rect.width, size.y);
	}

	/**
	 * Returns the arcFigure figure.
	 */
	protected Figure getFigure() {
		return arcFigure;
	}

	/**
	 * Sets the Arc figure.
	 * 
	 * @param arcFigure
	 *            The arcFigure to set
	 */
	protected void setFigure(ArcFigure arcFigure) {
		this.arcFigure = arcFigure;
	}

}
