package roneditor.gef.tools.objectnet;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import roneditor.gef.figures.objectnet.TransitionFigure;

/**
 * The {@link CellEditorLocator} for DirectEdit operations of the
 * {@link ONTransitionEditPart}. Determines the location of the DirectEdit
 * {@link CellEditor}.
 */
public class ONTransitionCellEditorLocator implements CellEditorLocator {
	private TransitionFigure transitionFigure;

	public ONTransitionCellEditorLocator(TransitionFigure placeFigure) {
		setFigure(placeFigure);
	}

	/**
	 * Determine the position for the {@link CellEditor} based on the attached
	 * {@link TransitionFigure}.
	 */
	public void relocate(CellEditor celleditor) {
		Text text = (Text) celleditor.getControl();

		Rectangle rect = transitionFigure.getClientArea();
		transitionFigure.translateToAbsolute(rect);

		org.eclipse.swt.graphics.Point size = text.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);

		org.eclipse.swt.graphics.Rectangle trim = text.computeTrim(0, 0, 0, 0);
		rect.translate(trim.x, trim.y);
		rect.width += trim.width + 10;
		rect.height += trim.height;

		rect.y += rect.height - size.y;
		text.setBounds(rect.x, rect.y, rect.width, size.y);
	}

	/**
	 * Returns the placeFigure figure.
	 */
	protected TransitionFigure setFigure() {
		return transitionFigure;
	}

	/**
	 * Sets the Place figure.
	 * 
	 * @param placeFigure
	 *            The placeFigure to set
	 */
	protected void setFigure(TransitionFigure placeFigure) {
		this.transitionFigure = placeFigure;
	}

}
