package roneditor.gef.tools.ron;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

import roneditor.gef.editparts.objectnet.ONPlaceEditPart;
import roneditor.gef.figures.objectnet.PlaceFigure;
import roneditor.gef.figures.ron.HLPlaceFigure;

/**
 * The {@link CellEditorLocator} for DirectEdit operations of the
 * {@link ONPlaceEditPart}. Determines the location of the DirectEdit
 * {@link CellEditor}.
 */
public class HLPlaceCellEditorLocator implements CellEditorLocator {
	private HLPlaceFigure placeFigure;

	public HLPlaceCellEditorLocator(HLPlaceFigure placeFigure) {
		setFigure(placeFigure);
	}

	/**
	 * Determine the position for the {@link CellEditor} based on the attached
	 * {@link PlaceFigure}.
	 */
	public void relocate(CellEditor celleditor) {
		Text text = (Text) celleditor.getControl();

		Rectangle rect = placeFigure.getLabelClientArea();
		placeFigure.translateToAbsolute(rect);

		org.eclipse.swt.graphics.Point size = text.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);

		org.eclipse.swt.graphics.Rectangle trim = text.computeTrim(0, 0, 0, 0);
		rect.translate(trim.x, trim.y);
		rect.width += trim.width + 10;
		rect.height += trim.height;

//		rect.y += rect.height - size.y + 5;
		text.setBounds(rect.x, rect.y, rect.width, size.y);
	}

	/**
	 * Returns the placeFigure figure.
	 */
	protected HLPlaceFigure getFigure() {
		return placeFigure;
	}

	/**
	 * Sets the Place figure.
	 * 
	 * @param placeFigure
	 *            The placeFigure to set
	 */
	protected void setFigure(HLPlaceFigure placeFigure) {
		this.placeFigure = placeFigure;
	}

}
