package roneditor.model;

import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;

/**
 * This class implements the CreationFactory used by the CreationTool. It uses
 * the EMF-generated factories to create the model instances.
 * 
 * @author Tony Modica
 */
public class ModelCreationFactory implements CreationFactory {

	final private Class<?> targetClass;

	public ModelCreationFactory(final Class<?> targetClass) {
		this.targetClass = targetClass;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Object result = null;
		Map registry = EPackage.Registry.INSTANCE;
		String objectnetModelURI = RonmodelPackage.eNS_URI;
		RonmodelPackage networkPackage = (RonmodelPackage) registry.get(objectnetModelURI);
		RonmodelFactory factory = networkPackage.getRonmodelFactory();
		if (targetClass.equals(ONTransition.class)) {
			result = factory.createONTransition();
		} else if (targetClass.equals(ONPlace.class)) {
			result = factory.createONPlace();
		} else if (targetClass.equals(ONArc.class)) {
			result = factory.createONArc();
		} else if (targetClass.equals(Mapping.class)) {
			result = factory.createMapping();
		} else if (targetClass.equals(HLPlace.class)) {
			result = factory.createHLPlace();
		} else if (targetClass.equals(HLArc.class)) {
			result = factory.createHLArc();
		} else if (targetClass.equals(HLTransition.class)) {
			result = factory.createHLTransition();
		}
		Assert.isNotNull(result, "ModelCreationFactory could not create a model!");
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}
}
