package roneditor.model;

import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * This Class provides some basic (static) operations for {@link ONTransition}
 * not to be found in the generated model.
 */
public class ONTransitionUtil {

	/**
	 * Check if the given {@link ONTransition} can fire. Hence if all source
	 * places have enough tokens and if all target places have sufficient
	 * capacity.
	 * 
	 * @param ONTansition
	 *            to check.
	 * @return True if the ONTransition can fire.
	 */
	public static boolean transitionCanFire(ONTransition transition) {
		// Check if all source places have enough tokens
		for (Object incomingArcObject : transition.getIncoming()) {
			final ONArc incomingArc = (ONArc) incomingArcObject;
			if ( incomingArc == null ) return false;
			final ONPlace sourcePlace = (ONPlace) incomingArc.getSource();
			if ( sourcePlace == null ) return false;
			if (sourcePlace.getTokens() < incomingArc.getWeight()) {
				return false;
			}
		}

		// Check if the capacity of all target places is sufficient
		for (Object outgoingArcObject : transition.getOutgoing()) {
			final ONArc outgoingArc = (ONArc) outgoingArcObject;
			if ( outgoingArc == null ) return false;
			final ONPlace targetPlace = (ONPlace) outgoingArc.getTarget();
			if ( targetPlace == null ) return false;
			int newCapacity = outgoingArc.getWeight() + targetPlace.getTokens();
			if (targetPlace.getCapacity() != 0 && targetPlace.getCapacity() < newCapacity) {
				return false;
			}
		}
		return true;
	}

}
