package roneditor.ui.contextmenu;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.CopyModelToClipboardAction;
import roneditor.gef.actions.PasteModelAction;
import roneditor.gef.actions.ron.ApplyRuleDialogAction;
import roneditor.gef.actions.ron.ApplyRulesAction;
import roneditor.gef.actions.ron.CreateHLPlaceAction;
import roneditor.gef.actions.ron.CreateTokenAction;
import roneditor.gef.actions.ron.FireDialogAction;

/**
 * This is used to populate the context menu for the TreeViewer in the
 * RONEditor.
 * 
 * @author Tony Modica
 */
public class RONEditorContextMenuProvider extends ContextMenuProviderWithActionRegistry {

	public RONEditorContextMenuProvider(final EditPartViewer viewer, final ActionRegistry actionRegistry) {
		super(viewer, actionRegistry);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void buildContextMenu(final IMenuManager menuManager) {
		GEFActionConstants.addStandardActionGroups(menuManager);

		// Actions in UNDO group
		String group = GEFActionConstants.GROUP_UNDO;
		appendActionToGroup(menuManager, ActionFactory.UNDO.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.REDO.getId(), group);

		// Actions in EDIT group
		group = GEFActionConstants.GROUP_EDIT;
		dynamicAppendActionToGroup(menuManager, GEFActionConstants.DIRECT_EDIT, group);
		appendActionToGroup(menuManager, ActionFactory.DELETE.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.COPY.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.CUT.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.PASTE.getId(), group);

		// Actions in REST group
		group = GEFActionConstants.GROUP_REST;
		dynamicAppendActionToGroup(menuManager, CreateHLPlaceAction.ID, group);
		dynamicAppendActionToGroup(menuManager, CreateTokenAction.ID, group);
		dynamicAppendActionToGroup(menuManager, ApplyRulesAction.ID, group);
		
		//Copy/Paste
		dynamicAppendActionToGroup(menuManager, CopyModelToClipboardAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, PasteModelAction.ID, GEFActionConstants.GROUP_VIEW);

		// Match and Fire Dialog
		dynamicAppendActionToGroup(menuManager, FireDialogAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, ApplyRuleDialogAction.ID, GEFActionConstants.GROUP_VIEW);

		
	}
}
