package roneditor.ui.contextmenu;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.CopyModelToClipboardAction;
import roneditor.gef.actions.PasteModelAction;
import roneditor.gef.actions.rule.CopyLHSToNACAction;
import roneditor.gef.actions.rule.CopyLHSToRHSAction;
import roneditor.gef.actions.rule.CopyRHSToNACAction;
import roneditor.gef.actions.rule.CreateNACAction;
import roneditor.gef.actions.rule.DeleteNACAction;
import roneditor.gef.actions.rule.SwitchNACAction;

/**
 * This is used to populate the context menu for the Viewers in the
 * {@link RuleNetPage}.
 * 
 * @author Tony Modica
 */
public class RulePageContextMenuProvider extends ContextMenuProviderWithActionRegistry {

	public RulePageContextMenuProvider(final EditPartViewer viewer, final ActionRegistry actionRegistry) {
		super(viewer, actionRegistry);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void buildContextMenu(final IMenuManager menuManager) {
		GEFActionConstants.addStandardActionGroups(menuManager);

		// Actions in UNDO group
		String group = GEFActionConstants.GROUP_UNDO;
		appendActionToGroup(menuManager, ActionFactory.UNDO.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.REDO.getId(), group);

		// Actions in EDIT group
		group = GEFActionConstants.GROUP_EDIT;
		dynamicAppendActionToGroup(menuManager, GEFActionConstants.DIRECT_EDIT, group);
		appendActionToGroup(menuManager, ActionFactory.DELETE.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.COPY.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.CUT.getId(), group);
		appendActionToGroup(menuManager, ActionFactory.PASTE.getId(), group);

		// Sample Action to switch/create/delete NACs
		dynamicAppendActionToGroup(menuManager, SwitchNACAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, CreateNACAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, DeleteNACAction.ID, GEFActionConstants.GROUP_VIEW);

		// Copies
		dynamicAppendActionToGroup(menuManager, CopyLHSToRHSAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, CopyRHSToNACAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, CopyLHSToNACAction.ID, GEFActionConstants.GROUP_VIEW);

		//Copy/Paste
		dynamicAppendActionToGroup(menuManager, CopyModelToClipboardAction.ID, GEFActionConstants.GROUP_VIEW);
		dynamicAppendActionToGroup(menuManager, PasteModelAction.ID, GEFActionConstants.GROUP_VIEW);

	}
}
