package roneditor.ui.dialog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Tool;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackEvent;
import org.eclipse.gef.commands.CommandStackEventListener;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.ui.PlatformUI;

import roneditor.gef.commands.objectnet.ObjectNetEditPartGraphLayoutCommand;
import roneditor.gef.editparts.objectnet.ObjectNetEditPart;
import roneditor.model.RuleSimulator;
import roneditor.model.RuleUtil;
import ronmodel.Codomain;
import ronmodel.HLPlace;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;

/**
 * This code was edited or generated using CloudGarden's Jigloo SWT/Swing GUI
 * Builder, which is free for non-commercial use. If Jigloo is being used
 * commercially (ie, by a corporation, company or business for any purpose
 * whatever) then you should purchase a license for each developer using Jigloo.
 * Please visit www.cloudgarden.com for details. Use of Jigloo implies
 * acceptance of these licensing terms. A COMMERCIAL LICENSE HAS NOT BEEN
 * PURCHASED FOR THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED LEGALLY FOR
 * ANY CORPORATE OR COMMERCIAL PURPOSE.
 */
abstract public class AbstractMatchDialog extends Dialog implements Adapter, CommandStackEventListener {

	/**
	 * A switch to determine whether this dialog has been canceled.
	 */
	private boolean canceled = true;

	/**
	 * A switch to determine if computed completions that do not respect the
	 * actual partial match are being filtered out.
	 */
	private boolean isAllCompletionsMode = false;

	/**
	 * The rules that can be selected to be applied in this dialog.
	 */
	private final ArrayList<Rule> rules;

	/**
	 * The object nets that can be selected to apply rules on in this dialog.
	 */
	private final ArrayList<ObjectNet> objectNets;

	/**
	 * The edit domain handling the mapping tool and the graphical viewers.
	 */
	private EditDomain editDomain;

	/**
	 * An action registry mainly for holding undo and redo action.
	 */
	private ActionRegistry actionRegistry;

	/**
	 * A command stack for this dialog to perform undo and redo.
	 */
	private CommandStack commandStack;

	/**
	 * A viewer showing a copy of the current rule's {@link LHS}.
	 */
	private GraphicalViewer lhsViewer;

	/**
	 * A viewer showing a copy of the current object net.
	 */
	private GraphicalViewer codomainViewer;

	/**
	 * A cache for the match between the current rule and object net that has
	 * been set in the viewers.
	 */
	private HashMap<ONNode, ONNode> matchToApply;

	/**
	 * The rule of whose {@link LHS} a copy is being shown in the lhsViewer.
	 */
	private Rule currentRule;

	/**
	 * A copy of the currently selected object net.
	 */
	private RHS currentObjectNetCopy;

	/**
	 * The position of the current object net in the objectNets list. This is
	 * stored because we are working on copies.
	 */
	private int currentObjectNetPos;

	/**
	 * If this is <code>null</code> we are in match completion mode! Otherwise
	 * it hold the (filtered) completions returned by AGG.
	 */
	private List<HashMap<ONNode, ONNode>> matchCompletions;

	/**
	 * The position of the current match completion in the matchCompletions
	 * list. This is stored to be able to restore the right completion with undo
	 * and redo.
	 */
	int currentCompletionPos;

	/**
	 * The high level place which hosts the current object net.
	 */
	private HLPlace hlPlace;

	/**
	 * A rule that has a copy of the currentRule's {@link LHS} as {@link LHS}
	 * and the CurrentObjectNetCopy as {@link RHS}. Its purpose is to ease the
	 * integration of the MatchDialog to existing {@link RONEditor}s,
	 * especially to display mappings like in an ordinary rule view.
	 */
	private Rule containerRule;

	// Visual stuff
	private Shell dialogShell;
	private Text textMessage;
	private Button buttonAllCompletions;
	private Button buttonCancel;
	private Button buttonApplyRule;
	private Composite compositeApplyLower;
	private Button buttonNextMatch;
	private Button buttonPreviousMatch;
	private Composite compositeMatchMiddle;
	private Button buttonNextNet;
	private Button buttonPreviousNet;
	private Composite compositeNetMiddle;
	private Composite compositeMiddle;
	private SashForm sashForm;
	private Button buttonNextRule;
	private Button buttonPreviousRule;
	private Composite compositeRulesUpper;
	private ToolBar toolBar;
	private ToolBarManager toolBarManager;
	private EditPartFactory editPartFactory;

	/**
	 * Needed for {@link Adapter} implementation.
	 */
	private Notifier target;

	/**
	 * Main constructor for this dialog.
	 * 
	 * @param rules
	 *            The rules that can be switched through for match-setting. Must
	 *            contain at least 1 rule.
	 * @param objectNets
	 *            The target object nets that can be switched through for
	 *            match-setting. Must contain at least 1 object net.
	 */
	public AbstractMatchDialog(final ArrayList<Rule> rules, final ArrayList<ObjectNet> objectNets) {
		super(new Shell(Display.getDefault()), SWT.NULL);
		Assert.isTrue(rules != null && !rules.isEmpty(), "MatchDialog requires at least one rule!");
		Assert.isTrue(objectNets != null && !objectNets.isEmpty(), "MatchDialog requires at least one object net!");
		this.rules = rules;
		this.objectNets = objectNets;
	}

	/**
	 * Builds the dialog by creating the widget hierarchy and calling
	 * {@link #init()}.
	 * 
	 * @see #init()
	 */
	public void open() {
		try {
			final Shell parent = getParent();
			dialogShell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE);

			{
				// Register as a resource user - SWTResourceManager will
				// handle the obtaining and disposing of resources
				SWTResourceManager.registerResourceUser(dialogShell);
			}

			final GridLayout dialogShellLayout = new GridLayout();
			dialogShell.setLayout(dialogShellLayout);

			dialogShell.setText("Specify match for rule application");
			// START >> compositeRulesUpper
			final GridData compositeUpperLData = new GridData();
			compositeUpperLData.grabExcessHorizontalSpace = true;
			compositeRulesUpper = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeUpperLayout = new GridLayout();
			compositeUpperLayout.numColumns = 2;
			compositeUpperLayout.makeColumnsEqualWidth = true;
			compositeRulesUpper.setLayout(compositeUpperLayout);
			compositeRulesUpper.setLayoutData(compositeUpperLData);
			// START >> buttonPreviousRule
			buttonPreviousRule = new Button(compositeRulesUpper, SWT.PUSH | SWT.CENTER);
			final GridData buttonPreviousRuleLData = new GridData();
			buttonPreviousRuleLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousRule.setLayoutData(buttonPreviousRuleLData);
			buttonPreviousRule.setText("Previous Rule");
			buttonPreviousRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousRuleMouseUp(evt);
				}
			});
			// END << buttonPreviousRule
			// START >> buttonNextRule
			buttonNextRule = new Button(compositeRulesUpper, SWT.PUSH | SWT.CENTER);
			final GridData buttonNextRuleLData = new GridData();
			buttonNextRuleLData.horizontalAlignment = GridData.CENTER;
			buttonNextRule.setLayoutData(buttonNextRuleLData);
			buttonNextRule.setText("Next Rule");
			buttonNextRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextRuleMouseUp(evt);
				}
			});
			// END << buttonNextRule
			// END << compositeRulesUpper
			// START >> sashForm
			sashForm = new SashForm(dialogShell, SWT.BORDER | SWT.VERTICAL);
			final FillLayout sashFormLayout = new FillLayout(org.eclipse.swt.SWT.HORIZONTAL);
			sashForm.setLayout(sashFormLayout);
			sashForm.setBackground(getParent().getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
			final GridData sashFormLData = new GridData();
			sashFormLData.verticalAlignment = GridData.FILL;
			sashFormLData.horizontalAlignment = GridData.FILL;
			sashFormLData.grabExcessHorizontalSpace = true;
			sashFormLData.grabExcessVerticalSpace = true;
			sashForm.setLayoutData(sashFormLData);
			sashForm.setForeground(getParent().getDisplay().getSystemColor(SWT.COLOR_LIST_FOREGROUND));
			// END << sashForm
			// START >> compositeMiddle
			final GridData compositeMiddleLData = new GridData();
			compositeMiddleLData.horizontalAlignment = GridData.FILL;
			compositeMiddleLData.grabExcessHorizontalSpace = true;
			compositeMiddle = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeMiddleLayout = new GridLayout();
			compositeMiddleLayout.numColumns = 2;
			compositeMiddle.setLayout(compositeMiddleLayout);
			compositeMiddle.setLayoutData(compositeMiddleLData);
			// START >> compositeNetMiddle
			final GridData compositeNetMiddleLData = new GridData();
			compositeNetMiddleLData.grabExcessHorizontalSpace = true;
			compositeNetMiddle = new Composite(compositeMiddle, SWT.NONE);
			final GridLayout compositeNetMiddleLayout = new GridLayout();
			compositeNetMiddleLayout.numColumns = 2;
			compositeNetMiddleLayout.makeColumnsEqualWidth = true;
			compositeNetMiddle.setLayout(compositeNetMiddleLayout);
			compositeNetMiddle.setLayoutData(compositeNetMiddleLData);
			// START >> buttonPreviousNet
			buttonPreviousNet = new Button(compositeNetMiddle, SWT.PUSH | SWT.CENTER);
			final GridData buttonPreviousNetLData = new GridData();
			buttonPreviousNetLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousNet.setLayoutData(buttonPreviousNetLData);
			buttonPreviousNet.setText("Previous Object Net");
			buttonPreviousNet.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousNetMouseUp(evt);
				}
			});
			// END << buttonPreviousNet
			// START >> buttonNextNet
			buttonNextNet = new Button(compositeNetMiddle, SWT.PUSH | SWT.CENTER);
			final GridData buttonNextNetLData = new GridData();
			buttonNextNetLData.horizontalAlignment = GridData.CENTER;
			buttonNextNet.setLayoutData(buttonNextNetLData);
			buttonNextNet.setText("Next Object Net");
			buttonNextNet.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextNetMouseUp(evt);
				}
			});
			// END << buttonNextNet
			// END << compositeNetMiddle
			// START >> compositeMatchMiddle
			final GridData composite1LData = new GridData();
			composite1LData.horizontalAlignment = GridData.END;
			compositeMatchMiddle = new Composite(compositeMiddle, SWT.NONE);
			final GridLayout composite1Layout = new GridLayout();
			composite1Layout.numColumns = 3;
			compositeMatchMiddle.setLayout(composite1Layout);
			compositeMatchMiddle.setLayoutData(composite1LData);
			// START >> buttonPreviousMatch
			buttonPreviousMatch = new Button(compositeMatchMiddle, SWT.PUSH | SWT.CENTER);
			final GridData buttonPreviousMatchLData = new GridData();
			buttonPreviousMatchLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousMatch.setLayoutData(buttonPreviousMatchLData);
			buttonPreviousMatch.setText("Previous Match");
			buttonPreviousMatch.setEnabled(false);
			buttonPreviousMatch.setToolTipText("Show previous match found by AGG");
			buttonPreviousMatch.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousMatchMouseUp();
				}
			});
			// END << buttonPreviousMatch

			createToolBar();

			// START >> buttonNextMatch
			buttonNextMatch = new Button(compositeMatchMiddle, SWT.PUSH | SWT.CENTER);
			final GridData buttonFindMatchLData = new GridData();
			buttonFindMatchLData.horizontalAlignment = GridData.CENTER;
			buttonNextMatch.setLayoutData(buttonFindMatchLData);
			buttonNextMatch.setText("Calculate Completions");
			buttonNextMatch.setEnabled(true);
			buttonNextMatch.setToolTipText("Show next match found by AGG");
			buttonNextMatch.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextMatchMouseUp();
				}
			});
			// END << buttonNextMatch
			// END << compositeMatchMiddle
			// END << compositeMiddle
			// START >> compositeApplyLower
			final GridData compositeLData = new GridData();
			compositeLData.verticalAlignment = GridData.END;
			compositeLData.horizontalAlignment = GridData.CENTER;
			compositeLData.grabExcessHorizontalSpace = true;
			compositeApplyLower = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeLayout = new GridLayout();
			compositeLayout.numColumns = 3;
			compositeLayout.makeColumnsEqualWidth = true;
			compositeApplyLower.setLayout(compositeLayout);
			compositeApplyLower.setLayoutData(compositeLData);
			// START >> buttonApplyRule
			buttonApplyRule = new Button(compositeApplyLower, SWT.PUSH | SWT.CENTER);
			final GridData buttonApplyRuleLData = new GridData();
			buttonApplyRuleLData.horizontalAlignment = GridData.CENTER;
			buttonApplyRule.setLayoutData(buttonApplyRuleLData);
			buttonApplyRule.setText("Apply Rule");
			buttonApplyRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonApplyRuleMouseUp();
				}
			});
			// END << buttonApplyRule
			// START >> buttonCancel
			buttonCancel = new Button(compositeApplyLower, SWT.PUSH | SWT.CENTER);
			final GridData buttonCancelLData = new GridData();
			buttonCancelLData.horizontalAlignment = GridData.CENTER;
			buttonCancel.setLayoutData(buttonCancelLData);
			buttonCancel.setText("Cancel");
			buttonCancel.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonCancelMouseUp();
				}
			});
			// END << buttonCancel
			// START >> buttonAllCompletions
			buttonAllCompletions = new Button(compositeApplyLower, SWT.CHECK | SWT.LEFT);
			buttonAllCompletions.setText("All Completions");
			buttonAllCompletions
					.setToolTipText("Unset this if a partial match should be considered when computing match completions.");
			buttonAllCompletions.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonAllCompletionsMouseUp();
				}
			});
			// END << buttonAllCompletions
			// END << compositeApplyLower
			// START >> textMessage
			textMessage = new Text(dialogShell, SWT.MULTI | SWT.WRAP);
			final GridData textMessageLData = new GridData();
			textMessageLData.verticalAlignment = GridData.END;
			textMessageLData.horizontalAlignment = GridData.CENTER;
			textMessageLData.grabExcessHorizontalSpace = true;
			textMessage.setLayoutData(textMessageLData);
			textMessage.setEditable(false);
			textMessage.setBackground(SWTResourceManager.getColor(236, 233, 216));
			textMessage.setForeground(SWTResourceManager.getColor(255, 0, 0));
			// END << textMessage

			// set up the rest
			init();

			dialogShell.layout();
			dialogShell.pack();
			dialogShell.open();
			
			performGraphLayout();
			
			final Display display = dialogShell.getDisplay();
			while (!dialogShell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Perform a GraphLayout to make sure everything is layed out correctly
	 */
	private void performGraphLayout() {
		(new ObjectNetEditPartGraphLayoutCommand( (ObjectNetEditPart)(lhsViewer.getContents())) ).execute();
		(new ObjectNetEditPartGraphLayoutCommand( (ObjectNetEditPart)(codomainViewer.getContents())) ).execute();
	}

	/**
	 * This method creates a tool bar with special undo and redo actions working
	 * on te dialog's command stack.
	 */
	private void createToolBar() {
		toolBarManager = new ToolBarManager(SWT.HORIZONTAL);
		toolBar = toolBarManager.createControl(compositeMatchMiddle);
		final GridData toolBarLData = new GridData();
		toolBarLData.horizontalAlignment = GridData.CENTER;
		toolBar.setLayoutData(toolBarLData);
		actionRegistry = new ActionRegistry();
		IAction action = new UndoAction(null) {
			@Override
			protected CommandStack getCommandStack() {
				return AbstractMatchDialog.this.getCommandStack();
			}
		};
		toolBarManager.add(action);
		actionRegistry.registerAction(action);
		action = new RedoAction(null) {
			@Override
			protected CommandStack getCommandStack() {
				return AbstractMatchDialog.this.getCommandStack();
			}
		};
		toolBarManager.add(action);
		actionRegistry.registerAction(action);
		toolBarManager.update(false);
	}

	/**
	 * Initialize some fields, create graphical viewers and container rule, set
	 * first rule and object net, configure buttons' status and reset match
	 * completion related stuff.
	 */
	private void init() {
		// initialize some fields
		matchToApply = new HashMap<ONNode, ONNode>();
		lhsViewer = createGraphicalViewer();
		codomainViewer = createGraphicalViewer();
		containerRule = RonmodelFactory.eINSTANCE.createRule();

		// set first rule and first net
		setCurrentRule(0);
		setCurrentCodomain(0);

		// maybe an empty match is applicable?
		buttonApplyRule.setEnabled(checkCurrentMatch());

		// allow switching only if multiple objects are available
		buttonNextRule.setEnabled(rules.size() > 1);
		buttonPreviousRule.setEnabled(rules.size() > 1);
		buttonNextNet.setEnabled(objectNets.size() > 1);
		buttonPreviousNet.setEnabled(objectNets.size() > 1);

		// initialize match completion buttons and mechanism
		resetMatchCompletion();
	}

	/**
	 * @return <code>true</code> if the match currently defined between the
	 *         actually shown {@link LHS} and {@link ObjectNet} is complete and
	 *         applicable.
	 * 
	 * @see #cacheCurrentMatch()
	 * @see #isMatchToApplyComplete()
	 */
	private boolean checkCurrentMatch() {
		cacheCurrentMatch();
		boolean isApplicable = false;
		if (isMatchToApplyComplete()) {
			isApplicable = (new RuleSimulator()).isApplicable(matchToApply, currentRule, currentObjectNetCopy);
		}
		return isApplicable;
	}

	/**
	 * Helper method for {@link #checkCurrentMatch()}.
	 * 
	 * @return <code>true</code> if for each {@link ONNode} in the current
	 *         {@link LHS} a {@link Mapping} exists.
	 */
	private boolean isMatchToApplyComplete() {
		for (final Object elem : containerRule.getLhs().getNodes()) {
			if (!matchToApply.containsKey(elem)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return A newly created and configured {@link ScrollingGraphicalViewer}
	 *         on the sashForm.
	 * 
	 * @see #getEditDomain()
	 * @see #getEditPartFactory()
	 */
	private GraphicalViewer createGraphicalViewer() {
		// set the new viewer on the viewer composite
		final GraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.createControl(sashForm);
		viewer.setEditDomain(getEditDomain());
		getEditDomain().addViewer(viewer);
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(getEditPartFactory());
		viewer.setContextMenu(createContextMenuProvider(viewer, actionRegistry));

		// set antialias on connection layer, this may be turned off if
		// causing trouble
		((ConnectionLayer) rootEditPart.getLayer(LayerConstants.CONNECTION_LAYER)).setAntialias(SWT.ON);
		// some optional coloring
		viewer.getControl().setBackground(PlatformUI.getWorkbench().getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		viewer.getControl().setForeground(PlatformUI.getWorkbench().getDisplay().getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		return viewer;
	}

	/**
	 * @param viewer
	 * @param actionRegistry2
	 * @return
	 */
	abstract protected MenuManager createContextMenuProvider(GraphicalViewer viewer, ActionRegistry actionRegistry2);

	/**
	 * @return A lazily created {@link EditDomain} that works on this dialog's
	 *         command stack with a {@link Tool} for creating Mappings as
	 *         default tool.
	 * 
	 * @see #getCommandStack()
	 */
	private EditDomain getEditDomain() {
		if (null == editDomain) {
			editDomain = new EditDomain();
			editDomain.setDefaultTool(createMappingTool());
			editDomain.loadDefaultTool();
			editDomain.setCommandStack(getCommandStack());
		}
		return editDomain;
	}

	/**
	 * @return
	 */
	abstract protected Tool createMappingTool();

	/**
	 * @return An lazily created {@link EditPartFactory} for the
	 *         {@link GraphicalViewer}s created in
	 *         {@link #createGraphicalViewer()}.
	 */
	private EditPartFactory getEditPartFactory() {
		if (editPartFactory == null) {
			editPartFactory = createEditPartFactory();
		}
		return editPartFactory;
	}

	/**
	 * @return
	 */
	abstract protected EditPartFactory createEditPartFactory();

	/**
	 * @return A lazily created {@link CommandStack} to which the dialog is
	 *         listening.
	 * 
	 * @see #stackChanged(CommandStackEvent)
	 */
	CommandStack getCommandStack() {
		if (commandStack == null) {
			commandStack = new CommandStack();
			commandStack.addCommandStackEventListener(this);
		}
		return commandStack;
	}

	/**
	 * @param rulePos
	 */
	private void setCurrentRule(final int rulePos) {
		if (currentObjectNetCopy != null) {
			undoCommandStack();
		}
		currentRule = rules.get(rulePos);
		containerRule = (Rule) EcoreUtil.copy(currentRule);
		containerRule.setName("foobarroole");
		containerRule.getLhs().setName(containerRule.getName());

		// delete all mappings to RHS
		for (final Object elem : new Vector(containerRule.getRhs().getMappings())) {
			final Mapping mapping = (Mapping) elem;
			mapping.setSource(null);
			mapping.setTarget(null);
			mapping.setCodomain(null);
		}
		
		// delete all nac mappings
		for (final Object nacObject :  new Vector(containerRule.getNacs())) {
			final NAC nac = (NAC) nacObject;
			for (final Object elem : new Vector(nac.getMappings())) {
				final Mapping mapping = (Mapping) elem;
				mapping.setSource(null);
				mapping.setTarget(null);
				mapping.setCodomain(null);
			}
			
		}
		RuleUtil.markColorMapDirty(containerRule);

		
		lhsViewer.setContents(containerRule.getLhs());
		lhsViewer.getContents().refresh();
		

		resetMatchCompletion();
	}

	/**
	 * @param objectNet
	 */
	private void setCurrentCodomain(final int objectNetPos) {
		if (currentObjectNetCopy != null) {
			undoCommandStack();
			currentObjectNetCopy.eAdapters().remove(this);
		}
		currentObjectNetPos = objectNetPos;
		final ObjectNet objectNet = objectNets.get(objectNetPos);
		hlPlace = objectNet.getPlace();
		final ObjectNet helperCopy = (ObjectNet) EcoreUtil.copy(objectNet);
		currentObjectNetCopy = RonmodelFactory.eINSTANCE.createRHS();
		currentObjectNetCopy.getArcs().addAll(helperCopy.getArcs());
		currentObjectNetCopy.setName(helperCopy.getName());
		currentObjectNetCopy.getNodes().addAll(helperCopy.getNodes());

		containerRule.setRhs(currentObjectNetCopy);
		currentObjectNetCopy.eAdapters().add(this);
		codomainViewer.setContents(currentObjectNetCopy);

		
		codomainViewer.getContents().refresh();
		
		resetMatchCompletion();
	}

	/**
	 * Undoes all actions on the command stack, flushes it and updates all
	 * actions. The current object net copy won't be notified about mapping
	 * changes as it will be deregistered as listener for model changes during
	 * execution of this method.
	 */
	private void undoCommandStack() {
		// deregister listener on codomain to speed up undo
		currentObjectNetCopy.eAdapters().remove(this);
		while (getCommandStack().canUndo()) {
			getCommandStack().undo();
		}
		// clear the command stack
		getCommandStack().flush();
		updateActions();
		// reregister listener on codomain
		currentObjectNetCopy.eAdapters().add(this);
	}

	/**
	 * Updates all actions in the action registry, mainly undo and redo.
	 */
	private void updateActions() {
		for (final Iterator it = actionRegistry.getActions(); it.hasNext();) {
			final IAction action = (IAction) it.next();
			if (action instanceof UpdateAction) {
				((UpdateAction) action).update();
			}
		}
	}

	/**
	 * 
	 */
	private void resetMatchCompletion() {
		buttonPreviousMatch.setText("Previous Match");
		buttonPreviousMatch.setEnabled(false);
		buttonNextMatch.setText("Calculate Completions");
		buttonNextMatch.setEnabled(true);
		compositeMatchMiddle.layout(true);
		compositeMatchMiddle.pack();
		matchCompletions = null;
		currentCompletionPos = -1;
	}

	/***************************************************************************
	 * Button handlers
	 **************************************************************************/

	/**
	 * 
	 */
	void buttonPreviousMatchMouseUp() {
		applyMatchCompletion(currentCompletionPos - 1);
	}

	/**
	 * 
	 */
	void buttonNextMatchMouseUp() {
		// get new match completions if mapping has been changed manually
		if (matchCompletions == null) {
			// get in match completion mode!
			matchCompletions = computeMatchCompletions();
			compositeMatchMiddle.pack();
			// is there any completion at all?
			if (matchCompletions.isEmpty()) {
				buttonNextMatch.setText("Next Match");
				buttonNextMatch.setEnabled(false);
				return;
			}
			// apply first match completion
			applyMatchCompletion(0);
		} else {
			// update enabled status of match buttons and set next match
			// completion
			applyMatchCompletion(currentCompletionPos + 1);
		}
	}

	private Vector<HashMap<ONNode, ONNode>> computeMatchCompletions() {
		cacheCurrentMatch();
		final Vector<HashMap<ONNode, ONNode>> allCompletions = (new RuleSimulator()).calculateMappings(matchToApply,
				containerRule, currentObjectNetCopy);
		// Determine result
		Vector<HashMap<ONNode, ONNode>> result;
		if (buttonAllCompletions.getSelection()) {
			result = allCompletions;
		} else {
			// Filter completions that don't include the current match
			result = new Vector<HashMap<ONNode, ONNode>>();
			for (final HashMap<ONNode, ONNode> completion : allCompletions) {
				// does completion contain current match?
				if (containsCurrentMatch(completion)) {
					result.add(completion);
				}
			}
		}
		return result;
	}

	/**
	 * 
	 */
	private void cacheCurrentMatch() {
		matchToApply.clear();
		for (final Object mapping : currentObjectNetCopy.getMappings()) {
			matchToApply.put(((Mapping) mapping).getSource(), ((Mapping) mapping).getTarget());
		}
	}

	private boolean containsCurrentMatch(final HashMap<ONNode, ONNode> completion) {
		for (final ONNode sourceNode : matchToApply.keySet()) {
			if (completion.get(sourceNode) != matchToApply.get(sourceNode)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @param matchCompletion
	 */
	@SuppressWarnings("unchecked")
	private void applyMatchCompletion(final int newCompletionPos) {
		// Get all Mappings in current codomain and add delete commands for them
		final CompoundCommand compoundCommand = new CompoundCommand() {
			// This is for the case, that the new completion overlaps with the
			// old
			// match. But as all mappings will be deleted the CompoundCommand
			// should be executable no matter what.
			@Override
			public boolean canExecute() {
				return true;
			}
		};
		for (final Object elem : currentObjectNetCopy.getMappings()) {
			final Mapping mapping = (Mapping) elem;
			compoundCommand.add(buildDeleteMappingCommand(mapping));
		}

		// create command for adding new match completion
		final HashMap<ONNode, ONNode> matchCompletion = matchCompletions.get(newCompletionPos);
		for (final ONNode node : matchCompletion.keySet()) {
			compoundCommand.add(buildCreateMappingCommand(currentObjectNetCopy, node, matchCompletion.get(node)));
		}

		// add simple command to keep the currentCompletionPos consistent
		final Command comm = new Command() {
			private int oldCompletionPos;

			@Override
			public void execute() {
				oldCompletionPos = currentCompletionPos;
				currentCompletionPos = newCompletionPos;
			}

			@Override
			public void undo() {
				currentCompletionPos = oldCompletionPos;
			}
		};
		compoundCommand.add(comm);

		// unregister EMF listener to speed up and run delete compound command
		currentObjectNetCopy.eAdapters().remove(this);
		getCommandStack().execute(compoundCommand);

		// reregister EMF listener
		currentObjectNetCopy.eAdapters().add(this);

		// as we have a completion, activate he apply button!
		buttonApplyRule.setEnabled(true);
		// update match buttons
		// not first completion?
		if (currentCompletionPos > 0) {
			buttonPreviousMatch.setEnabled(true);
			buttonPreviousMatch.setText("Previous Match" + "(" + currentCompletionPos + ")");
		} else {
			buttonPreviousMatch.setEnabled(false);
			buttonPreviousMatch.setText("Previous Match");
		}
		// reached end of match list?
		if (currentCompletionPos < matchCompletions.size() - 1) {
			buttonNextMatch.setEnabled(true);
			buttonNextMatch.setText("Next Match" + "(" + (matchCompletions.size() - 1 - currentCompletionPos) + ")");
		} else {
			buttonNextMatch.setEnabled(false);
			buttonNextMatch.setText("Next Match");
		}
		compositeMatchMiddle.layout(true);
		compositeMatchMiddle.pack();
	}

	/**
	 * @param mapping
	 * @return
	 */
	abstract protected Command buildDeleteMappingCommand(Mapping mapping);

	/**
	 * @param codomain
	 * @param source
	 * @param target
	 * @return
	 */
	abstract protected Command buildCreateMappingCommand(Codomain codomain, ONNode source, ONNode target);

	/**
	 * @param evt
	 */
	void buttonPreviousRuleMouseUp(@SuppressWarnings("unused")
	final MouseEvent evt) {
		final int pos = rules.indexOf(currentRule);
		final int newRulePos = pos == 0 ? rules.size() - 1 : pos - 1;
		setCurrentRule(newRulePos);
	}

	/**
	 * @param evt
	 */
	void buttonNextRuleMouseUp(@SuppressWarnings("unused")
	final MouseEvent evt) {
		final int pos = rules.indexOf(currentRule);
		final int newRulePos = pos == rules.size() - 1 ? 0 : pos + 1;
		setCurrentRule(newRulePos);
	}

	/**
	 * @param evt
	 */
	void buttonPreviousNetMouseUp(@SuppressWarnings("unused")
	final MouseEvent evt) {
		final int newNetPos = currentObjectNetPos == 0 ? objectNets.size() - 1 : currentObjectNetPos - 1;
		setCurrentCodomain(newNetPos);
	}

	/**
	 * @param evt
	 */
	void buttonNextNetMouseUp(@SuppressWarnings("unused")
	final MouseEvent evt) {
		final int newNetPos = currentObjectNetPos == objectNets.size() - 1 ? 0 : currentObjectNetPos + 1;
		setCurrentCodomain(newNetPos);
	}

	/**
	 * 
	 */
	void buttonCancelMouseUp() {
		closeDialog();
	}

	/**
	 * 
	 */
	void buttonApplyRuleMouseUp() {
		canceled = false;
		// store current match in a map
		cacheCurrentMatch();
		closeDialog();
	}

	/**
	 * 
	 */
	private void closeDialog() {
		// remove listener
		getCommandStack().removeCommandStackEventListener(this);
		currentObjectNetCopy.eAdapters().remove(this);

		// delete all mappings that have been created in this dialog so far
		while (getCommandStack().canUndo()) {
			getCommandStack().undo();
		}
		dialogShell.close();
	}

	/**
	 * Handler method for setting "All completions" switch.
	 */
	void buttonAllCompletionsMouseUp() {
		isAllCompletionsMode = !isAllCompletionsMode;
		buttonAllCompletions.setSelection(isAllCompletionsMode);
	}

	/**
	 * Updates all actions if the command stack changes.
	 * 
	 * @see org.eclipse.gef.commands.CommandStackEventListener#stackChanged(org.eclipse.gef.commands.CommandStackEvent)
	 * @see #updateActions()
	 */
	public void stackChanged(final CommandStackEvent event) {
		// update actions (undo, redo)
		updateActions();
	}

	/***************************************************************************
	 * Getters to retrieve information after dialog has been closed.
	 **************************************************************************/

	/**
	 * @return the match being cached 1. by pushing apply button 2. for current
	 *         match checking 3. for computing match completions
	 * 
	 * @see #cacheCurrentMatch()
	 */
	public HashMap<ONNode, ONNode> getMatchToApply() {
		return matchToApply;
	}

	/**
	 * @return the objectNetCopy
	 * 
	 * @see #setCurrentCodomain(int)
	 */
	public Codomain getObjectNetCopy() {
		return currentObjectNetCopy;
	}

	/**
	 * @return
	 */
	public ObjectNet getObjectNet() {
		return objectNets.get(currentObjectNetPos);
	}

	/**
	 * @return the currentRule
	 * 
	 * @see #setCurrentRule(int)
	 */
	public Rule getCurrentRule() {
		return currentRule;
	}

	/**
	 * @return The hlPlace that hosts the currently selected object net.
	 */
	public HLPlace getHLPlace() {
		return hlPlace;
	}

	/**
	 * @return <code>true</code> if this dialog has been canceled.
	 */
	public boolean isCanceled() {
		return canceled;
	}

	/***************************************************************************
	 * Adapter implementation
	 **************************************************************************/

	/**
	 * If a change of mappings happens on the current object net copy the
	 * "Apply" button will be set according to the current match and the match
	 * completion mechanism will be reset.
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 * @see #checkCurrentMatch()
	 * @see #resetMatchCompletion()
	 */
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.CODOMAIN__MAPPINGS:
			RuleUtil.markColorMapDirty(containerRule);

			buttonApplyRule.setEnabled(checkCurrentMatch());
			// reset match completion mechanism
			resetMatchCompletion();
			break;
		default:
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(final Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(final Object type) {
		return false;
	}
}
