package roneditor.ui.dialog;

import java.util.ArrayList;

import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Tool;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.MenuManager;

import roneditor.gef.commands.rule.DeleteMappingCommand;
import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.gef.tools.rule.RuleMappingCreationTool;
import roneditor.model.ModelCreationFactory;
import roneditor.ui.contextmenu.RulePageContextMenuProvider;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;

/**
 * Subclasses of AbstractMatchDialog are meant to adapt AbstractMatchDialog to a
 * concrete RONEditor by implementing the following methods.
 * 
 * @author Tony Modica
 */
public class MatchDialog extends AbstractMatchDialog {

	/**
	 * Main constructor for this dialog.
	 * 
	 * @param rules
	 *            The rules that can be switched through for match-setting. Must
	 *            contain at least 1 rule.
	 * @param objectNets
	 *            The target object nets that can be switched through for
	 *            match-setting. Must contain at least 1 object net.
	 */
	public MatchDialog(final ArrayList<Rule> rules,
			final ArrayList<ObjectNet> objectNets) {
		super(rules, objectNets);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createContextMenuProvider(org.eclipse.gef.GraphicalViewer,
	 *      org.eclipse.gef.ui.actions.ActionRegistry)
	 */
	@Override
	protected MenuManager createContextMenuProvider(
			final GraphicalViewer viewer, final ActionRegistry actionRegistry) {
		return new RulePageContextMenuProvider(viewer, actionRegistry);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createMappingTool()
	 */
	@Override
	protected Tool createMappingTool() {
		System.out.println("MatchDialog.createMappingTool()");
		return new RuleMappingCreationTool(new ModelCreationFactory(Mapping.class));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#buildCreateMappingCommand(ronmodel.Codomain,
	 *      ronmodel.ONNode, ronmodel.ONNode)
	 */
	@Override
	protected Command buildCreateMappingCommand(final Codomain codomain,
			final ONNode source, final ONNode target) {
		final UpdateMappingCommand createCommand = new UpdateMappingCommand();
		System.out.println("MatchDialog.buildCreateMappingCommand() codomain: "+codomain+ " source:"+source+ " target: "+target);
		createCommand.setCodomain(codomain);
		createCommand.setMapping(RonmodelFactory.eINSTANCE.createMapping());
		createCommand.setSource(source);
		createCommand.setTarget(target);
		createCommand.update();
		return createCommand;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#buildDeleteMappingCommand(ronmodel.Mapping)
	 */
	@Override
	protected Command buildDeleteMappingCommand(final Mapping mapping) {
		final DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
		deleteCommand.setMapping(mapping);
		return deleteCommand;
	}
}
