package roneditor.ui.part;

import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.IPageSite;

import roneditor.gef.actions.CopyModelToClipboardAction;
import roneditor.gef.actions.PasteModelAction;
import roneditor.gef.actions.objectnet.MoveNodeAction;
import roneditor.gef.actions.objectnet.ObjectNetEditPartGraphLayoutAction;
import roneditor.gef.actions.rule.CopyLHSToNACAction;
import roneditor.gef.actions.rule.CopyLHSToRHSAction;
import roneditor.gef.actions.rule.CopyRHSToNACAction;
import roneditor.gef.actions.rule.CreateNACAction;
import roneditor.gef.actions.rule.DeleteNACAction;
import roneditor.gef.actions.rule.SwitchNACAction;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.gef.palette.rule.RulePaletteRoot;
import roneditor.model.RuleUtil;
import roneditor.ui.contextmenu.ContextMenuProviderWithActionRegistry;
import roneditor.ui.contextmenu.RulePageContextMenuProvider;
import roneditor.ui.views.AbstractPageBookView;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link Rule}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetEditPartFactory} and
 * {@link RulePageContextMenuProvider}, configures the key-handler and creates
 * actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 */
public class RulePage extends OwnAbstractMultiViewerPage implements Adapter {

	NAC activeNAC;

	public RulePage(final Rule rule, final IEditorPart editor) {
		super(rule, editor);
	}

	@Override
	public void init(IPageSite pageSite) {
		super.init(pageSite);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
		ActionRegistry editorRegistry = (ActionRegistry) getEditor().getAdapter(ActionRegistry.class);

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);

		// A sample action to switch through the NACs
		final Action action = new SwitchNACAction(this);
		getActionRegistry().registerAction(action);

		final Action createNACAction = new CreateNACAction(this, getEditor());
		getActionRegistry().registerAction(createNACAction);

		final Action deleteNACAction = new DeleteNACAction(this, getEditor());
		getActionRegistry().registerAction(deleteNACAction);

		final Action copyLhsRhsAction = new CopyLHSToRHSAction(this, getEditor());
		getActionRegistry().registerAction(copyLhsRhsAction);
		final Action copyLhsNACAction = new CopyLHSToNACAction(this, getEditor());
		getActionRegistry().registerAction(copyLhsNACAction);
		final Action copyRhsNACAction = new CopyRHSToNACAction(this, getEditor());
		getActionRegistry().registerAction(copyRhsNACAction);

		getActionRegistry().registerAction(editorRegistry.getAction(ObjectNetEditPartGraphLayoutAction.ID));
		registerHandleForEditorAction(ObjectNetEditPartGraphLayoutAction.ID);

		getActionRegistry().registerAction(editorRegistry.getAction(ActionFactory.DELETE.getId()));
		registerHandleForEditorAction(ActionFactory.DELETE.getId());

		final Action copyObjectNetToClipboardAction = new CopyModelToClipboardAction(getEditor());
		getActionRegistry().registerAction(copyObjectNetToClipboardAction);
		registerHandleForEditorAction(ActionFactory.COPY.getId());

		final Action pasteObjectNetAction = new PasteModelAction(getEditor());
		getActionRegistry().registerAction(pasteObjectNetAction);
		registerHandleForEditorAction(ActionFactory.PASTE.getId());

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry().getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry().getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry().getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry().getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		// Just show the nets like in ObjectNetView, nothing special here
		return new ObjectNetEditPartFactory(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(final EditPartViewer viewer) {
		return new RulePageContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// ObjectNetEditorPalette
		return new RulePaletteRoot();
	}

	/**
	 * Helper function to return all {@link Mapping} that are associated with
	 * the current {@link Rule}.
	 * 
	 * @return
	 */
	public List<Mapping> getAllMappings() {
		return RuleUtil.getAllMappings(getRule());
	}

	/**
	 * Check all morphisms for correctness, such as compatibility, injectivity
	 * and pre- and post-conditions.
	 */
	public void checkMorphisms() {
		roneditor.model.RuleChecker.isRuleOk(this.getEditor(), this.getRule());
	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/**
	 * The Adapter is registered to all critical areas for a rule, hence we
	 * react on model changes that need the morphisms be rechecked for validity
	 * or mark our color table for the mappings dirty.
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		int eventId = notification.getEventType();

		// Always check morphisms for now
		checkMorphisms();

		if (eventId == Notification.SET) {
			// On critical model changes we need to re-validate the morphisms
			// TODO: Please check if this is sufficient!
			switch (featureId) {
			case RonmodelPackage.ON_ARC__SOURCE:
			case RonmodelPackage.ON_ARC__TARGET:
			case RonmodelPackage.ON_ARC__WEIGHT:
			case RonmodelPackage.ON_PLACE__TOKENS:
				// checkMorphisms();
				break;
			}
		}

		// Always update color-map for now
		RuleUtil.markColorMapDirty(getRule());
/*
		// A new mapping is created, hence we mark the color map dirty
		if (( eventId == Notification.ADD || eventId == Notification.REMOVE )  && featureId == RonmodelPackage.CODOMAIN__MAPPINGS) {
			RuleUtil.markColorMapDirty(getRule());
		}
*/
		// ensure we have a visible NAC
		if ((eventId == Notification.ADD || eventId == Notification.REMOVE) && featureId == RonmodelPackage.RULE__NACS) {
			setViewersContents(0, getActiveNAC());
		}
	}

	/**
	 * A {@link RulePage} shows three nets: {@link NAC}, {@link LHS} and
	 * {@link RHS} of the {@link Rule}.
	 * 
	 * @see roneditor.ui.part.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 3;
	}

	/**
	 * A {@link RulePage} shows the first {@link NAC} of the {@link Rule} and
	 * the {@link LHS} and {@link RHS}.
	 * 
	 * @see roneditor.ui.part.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		final Rule rule = (Rule) getModel();
		LHS lhs = rule.getLhs();
		RHS rhs = rule.getRhs();
		return new EObject[] { getActiveNAC(), lhs, rhs };
	}

	/**
	 * @return The model of the page casted to a {@link Rule}.
	 */
	public Rule getRule() {
		return (Rule) getModel();
	}

	/**
	 * Change the contents of the viewer for the NACs (the left-most) to the
	 * next element in the list of the rule's NACs. If the last NAC is being
	 * shown switch to the first in the list.
	 */
	@SuppressWarnings("unchecked")
	public void setNextNAC() {
		final List<NAC> nacs = getRule().getNacs();
		final NAC newNAC;
		if (getActiveNAC() == null) {
			newNAC = null;
		} else {
			final int actualShowing = nacs.lastIndexOf(getActiveNAC());
			newNAC = nacs.get((actualShowing + 1) % nacs.size());
		}
		setActiveNAC(newNAC);
	}

	/**
	 * Return the active {@link NAC} with some initial checking. If we have no
	 * {@link NAC}s associated return <code>null</code>, if we have no
	 * active {@link NAC} yet, set the first one from the list of {@link NAC}s
	 * as active {@link NAC}. Return then active {@link NAC}.
	 * 
	 * @return <code>null</code> if there is no {@link NAC}, the active
	 *         {@link NAC} otherwise (the first one as default)
	 */
	public NAC getActiveNAC() {
		if (getRule().getNacs().size() == 0) {
			return null;
		} else if (activeNAC == null) {
			activeNAC = (NAC) getRule().getNacs().get(0);
		}
		return activeNAC;
	}

	public void setActiveNAC(NAC activeNAC) {
		this.activeNAC = activeNAC;
		setViewersContents(0, activeNAC);
	}
}
