package roneditor.ui.perspective;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;
import org.eclipse.ui.IPlaceholderFolderLayout;

import roneditor.IRonEditorConstants;

/**
 * This the default definition of the perspective for the RON editor.
 * 
 * @author Tony Modica
 */
public class RONEditorPerspective implements IPerspectiveFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IPerspectiveFactory#createInitialLayout(org.eclipse.ui.IPageLayout)
	 */
	public void createInitialLayout(final IPageLayout layout) {
		defineActions(layout);
		defineLayout(layout);
	}

	/**
	 * Defines the initial actions for a page.
	 * 
	 * @param layout
	 *            The layout we are filling
	 */
	private void defineActions(final IPageLayout layout) {
		layout.addNewWizardShortcut("roneditor.dialogs.RONFileCreationWizard"); //$NON-NLS-1$

		// Add "show views".
		layout.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);
		layout.addShowViewShortcut(IPageLayout.ID_PROBLEM_VIEW);
		layout.addShowViewShortcut("org.eclipse.pde.runtime.LogView");
		layout.addShowViewShortcut("RONEditor.NavigatorView");
	}

	/**
	 * Defines the initial perspective layout.
	 * 
	 * @param layout
	 *            The layout we are filling
	 */
	private void defineLayout(final IPageLayout layout) {

		final String editorArea = layout.getEditorArea();

		//final IFolderLayout topLeft = layout.createFolder("topLeft", IPageLayout.LEFT, (float) 0.2, editorArea);
		//topLeft.addView("RONEditor.NavigatorView");
		
		final IPlaceholderFolderLayout topRight = layout.createFolder("topRight", IPageLayout.RIGHT, (float) 0.2, editorArea);
		// * is for necessary for the different secondaryIDs of the Views
		topRight.addPlaceholder(IRonEditorConstants.objectNetViewID + ":*");
		topRight.addPlaceholder(IRonEditorConstants.ruleViewID + ":*");
		topRight.addPlaceholder(IRonEditorConstants.ronViewID + ":*");
		
		final IFolderLayout bottomRight = layout.createFolder("bottomRight", IPageLayout.BOTTOM, (float) 0.8,"topRight");
		bottomRight.addView(IPageLayout.ID_PROP_SHEET);
		bottomRight.addView("org.eclipse.pde.runtime.LogView");
	}
}