package roneditor.ui.views;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.IPage;

import roneditor.ui.part.ObjectNetPage;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * <p>
 * This class extends and implements the abstract class
 * {@link AbstractPageBookView} to display a model on an {@link ObjectNetPage}.
 * <br>
 * It is expected that the model object is a {@link ObjectNet} instance from the
 * {@link RonmodelPackage}.
 * </p>
 * 
 * <p>
 * If the EMF model notifies a change of the object net's name the views part
 * name is set to the value {@link #calculatePartName()} returns which is the
 * name of the model {@link ObjectNet}.
 * </p>
 * 
 * @author Tony Modica
 */
public class ObjectNetPageBookView extends AbstractPageBookView {

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageBookView#getPageForModel(java.lang.Object)
	 */
	@Override
	protected IPage getPageForModel(final EObject model) {
		Assert.isTrue(model instanceof ObjectNet,
				"The model that should be displayed in ObjectNetPageBookView is not an instance of the class ObjectNet!");
		return new ObjectNetPage((ObjectNet) model, (IEditorPart) getBootstrapPart());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.OBJECT_NET__NAME:
			setPartName(calculatePartName());
			break;
		default:
			break;
		}
	}

	/**
	 * @return The name of the model {@link ObjectNet}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageBookView#calculatePartName()
	 */
	@Override
	protected String calculatePartName() {
		return ((ObjectNet) getModel()).getName();
	}

}
