package roneditor.ui.views;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.IPage;

import roneditor.ui.part.RONPage;
import ronmodel.RON;
import ronmodel.RonmodelPackage;

/**
 * <p>
 * This class extends and implements the abstract class
 * {@link AbstractPageBookView} to display a model on an {@link RONPage}.
 * <br>
 * It is expected that the model object is a {@link RON} instance from the
 * {@link RonmodelPackage}.
 * </p>
 * 
 * <p>
 * If the EMF model notifies a change of the object net's name the views part
 * name is set to the value {@link #calculatePartName()} returns which is the
 * name of the model {@link ObjectNet}.
 * </p>
 * 
 * @author seb
 */
public class RONPageBookView extends AbstractPageBookView {

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageBookView#getPageForModel(java.lang.Object)
	 */
	@Override
	protected IPage getPageForModel(final EObject model) {
		Assert.isTrue(model instanceof RON,
				"The model that should be displayed in RONPageBookView is not an instance of the class RON!");
		return new RONPage((RON) model, (IEditorPart) getBootstrapPart());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.RON__NAME:
			setPartName(calculatePartName());
			break;
		default:
			break;
		}
	}

	/**
	 * @return The name of the model {@link RON}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageBookView#calculatePartName()
	 */
	@Override
	protected String calculatePartName() {
		return ((RON) getModel()).getName();
	}

}
