package roneditor.ui.views.properties;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONTransition;

/**
 * Property source for a Place.
 */
public class HLTransitionPropertySource implements IPropertySource {

	protected static final int NAME_ID = 0;

	protected static final int KIND_ID = 1;
	
	protected static final String NAME_LABEL = "Name";

	protected static final String KIND_LABEL = "Kind";

	
	private HLTransition hlTransition;

	public HLTransitionPropertySource(final HLTransition onTransition) {
		this.hlTransition = onTransition;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return hlTransition;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		final Collection<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
		descriptors.add(new TextPropertyDescriptor(NAME_ID, NAME_LABEL));
		String[] values = {"STANDARD", "APPLYRULE", "FIRE", "SPLIT"};
		descriptors.add(new ComboBoxPropertyDescriptor(KIND_ID, KIND_LABEL,values));
		return (IPropertyDescriptor[]) descriptors.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch ((int) ((Integer) id).intValue()) {
		case NAME_ID:
			return hlTransition.getName();
		case KIND_ID:
			return hlTransition.getKind().getValue();			
		default:
			return null;
		}
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object id) {
		return false;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySouce#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object id) {
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case NAME_ID:
			hlTransition.setName((String) value);
			break;
		case KIND_ID:
			hlTransition.setKind( HLTransitionKind.get( ((Integer)value).intValue()) );
		default:
		}
	}

}
