package roneditor.ui.views.properties;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;


/**
 * Property source for delta data.
 * 
 * @author Tony Modica
 */
public class LocationPropertySource implements IPropertySource {

	final public static int ID_X = 0;

	final public static int ID_Y = 1;

	final private static IPropertyDescriptor[] descriptors;

	static {
		// Input for the x- and y-Coordinates should be validated if it is a
		// number.
		final PropertyDescriptor xProp = new TextPropertyDescriptor(Integer.valueOf(ID_X), "x");
		xProp.setValidator(NumberCellEditorValidator.instance(false));
		final PropertyDescriptor yProp = new TextPropertyDescriptor(Integer.valueOf(ID_Y), "y");
		yProp.setValidator(NumberCellEditorValidator.instance(false));
		descriptors = new IPropertyDescriptor[] { xProp, yProp };
	}

	private Point location;

	public LocationPropertySource(final Point location) {
		this.location = location.getCopy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch (((Integer) id).intValue()) {
		case ID_X:
			return Integer.valueOf(location.x).toString();
		case ID_Y:
			return Integer.valueOf(location.y).toString();
		default:
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case ID_X:
			location.x = Integer.valueOf((String) value).intValue();
			break;
		case ID_Y:
			location.y = Integer.valueOf((String) value).intValue();
			break;
		default:
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return location.getCopy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return descriptors;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object propName) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object propName) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "Point(" + location.x + "," + location.y + ")";//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
	}

}
