package roneditor.ui.views.properties;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.ONArc;

/**
 * Property source for a Place.
 */
public class ONArcPropertySource implements IPropertySource {

	protected static final int NAME_ID = 0;

	protected static final int WEIGHT_ID = 1;

	protected static final String NAME_LABEL = "Name";

	protected static final String WEIGHT_LABEL = "Weight";

	private ONArc onArc;

	public ONArcPropertySource(final ONArc onArc) {
		this.onArc = onArc;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return onArc;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		final Collection<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
		descriptors.add(new TextPropertyDescriptor(NAME_ID, NAME_LABEL));
		descriptors.add(new TextPropertyDescriptor(WEIGHT_ID, WEIGHT_LABEL));
		return (IPropertyDescriptor[]) descriptors.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch ((int) ((Integer) id).intValue()) {
		case NAME_ID:
			return onArc.getName();
		case WEIGHT_ID:
			return new Integer(onArc.getWeight()).toString();
		default:
			return null;
		}
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object id) {
		return false;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySouce#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object id) {
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case NAME_ID:
			onArc.setName((String) value);
			break;
		case WEIGHT_ID:
			try {
				int i = new Integer((String) value);
				if (i > 0)
					onArc.setWeight(i);
			} catch (Exception e) {
				// TODO: handle exception
			}
			break;
		default:
		}
	}

}
