package roneditor.ui.views.properties;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.ONPlace;

/**
 * Property source for a Place.
 */
public class ONPlacePropertySource implements IPropertySource {

	protected static final int NAME_ID = 0;

	protected static final int TOKENS_ID = 1;

	protected static final int CAPACITY_ID = 2;

	protected static final String NAME_LABEL = "Name";

	protected static final String TOKENS_LABEL = "Tokens";

	protected static final String CAPACITY_LABEL = "Capacity";

	private ONPlace onPlace;

	public ONPlacePropertySource(final ONPlace place) {
		this.onPlace = place;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return onPlace;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		final Collection<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
		descriptors.add(new TextPropertyDescriptor(NAME_ID, NAME_LABEL));
		descriptors.add(new TextPropertyDescriptor(TOKENS_ID, TOKENS_LABEL));
		descriptors.add(new TextPropertyDescriptor(CAPACITY_ID, CAPACITY_LABEL));
		return (IPropertyDescriptor[]) descriptors.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch ((int) ((Integer) id).intValue()) {
		case NAME_ID:
			return onPlace.getName();
		case TOKENS_ID:
			return new Integer(onPlace.getTokens()).toString();
		case CAPACITY_ID:
			return new Integer(onPlace.getCapacity()).toString();
		default:
			return null;
		}
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object id) {
		return false;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySouce#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object id) {
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case NAME_ID:
			onPlace.setName((String) value);
			break;
		case TOKENS_ID:
			try {
				int i = new Integer((String) value);
				if (i >= 0 && (onPlace.getCapacity() >= i || onPlace.getCapacity() == 0))
					onPlace.setTokens(i);
			} catch (Exception e) {
				// TODO: handle exception
			}
			break;
		case CAPACITY_ID:
			try {
				int i = new Integer((String) value);
				if (i >= 0) {
					if (onPlace.getTokens() > i && i != 0)
						onPlace.setTokens(i);
					onPlace.setCapacity(i);
				}
			} catch (Exception e) {
				// TODO: handle exception
			}
			break;
		default:
		}
	}

}
