package roneditor.ui.views.properties;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.ONTransition;

/**
 * Property source for a Place.
 */
public class ONTransitionPropertySource implements IPropertySource {

	protected static final int NAME_ID = 0;

	protected static final String NAME_LABEL = "Name";

	private ONTransition onTransition;

	public ONTransitionPropertySource(final ONTransition onTransition) {
		this.onTransition = onTransition;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return onTransition;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		final Collection<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
		descriptors.add(new TextPropertyDescriptor(NAME_ID, NAME_LABEL));
		return (IPropertyDescriptor[]) descriptors.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch ((int) ((Integer) id).intValue()) {
		case NAME_ID:
			return onTransition.getName();
		default:
			return null;
		}
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object id) {
		return false;
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySouce#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object id) {
	}

	/*
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case NAME_ID:
			onTransition.setName((String) value);
			break;
		default:
		}
	}

}
