/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package roneditor.ui.wizard;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;

/**
 * The "New" wizard page is used in the RONFileCreationWizard and allows setting
 * the container for the new file as well as the file name. The page will only
 * accept file name without the extension OR with the extension that matches
 * ".ron".
 * 
 * @author Tony Modica
 */
// HINT Diese WizardPage k�nnt Ihr anpassen, um sie in einem Wizard zum Erzeugen
// von Modell-Files zu benutzen.
public class RONFileCreationPage extends WizardNewFileCreationPage {

	final private IWorkbench workbench;

	private static int eCount = 1;

	public RONFileCreationPage(final IWorkbench workbench, final IStructuredSelection selection) {
		super("RON File Creation Wizard", selection);
		setTitle("RONEditor File");
		setDescription("This wizard creates a new file with '.ron' extension that can be opened by the RONEditor.");
		// this.setImageDescriptor(ImageDescriptor.createFromFile(FlowPlugin.class,"images/flowbanner.gif"));
		this.workbench = workbench;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(final Composite parent) {
		super.createControl(parent);
		setFileName("RON" + eCount + ".ron");
		setPageComplete(validatePage());
	}

	public boolean finish() {
		final IFile newFile = createNewFile();
		if (newFile == null) {
			return false; // ie.- creation was unsuccessful
		}

		// Since the file resource was created fine, open it for editing
		try {
			final IWorkbenchPage page = workbench.getActiveWorkbenchWindow().getActivePage();
			if (page != null) {
				IDE.openEditor(page, newFile, true);
			}
		} catch (final org.eclipse.ui.PartInitException e) {
			e.printStackTrace();
			return false;
		}
		eCount++;
		return true;
	}
}