package roneditor.ui.wizard;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * This is a wizard to create a RONEditor file resource. The wizard uses a
 * RONFileCreationPage which creates one file with the extension "ron". If an
 * editor is registered for the same extension, it will open it.
 * 
 * @author Tony Modica
 */
// HINT Dieser Wizard ist in der plugin.xml unter Extensions
// (org.eclipse.ui.newWizards) registriert, damit er �ber File -> New.. zur
// Verf�gung steht!
public class RONFileCreationWizard extends Wizard implements INewWizard {

	private RONFileCreationPage page;

	private IStructuredSelection selection;

	private IWorkbench workbench;

	/**
	 * Adding the RONFileCreationPage to the wizard.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	@Override
	public void addPages() {
		page = new RONFileCreationPage(workbench, selection);
		addPage(page);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We
	 * will create an operation and run it using wizard as execution context.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		return page.finish();
	}

	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(final IWorkbench aWorkbench, final IStructuredSelection sel) {
		workbench = aWorkbench;
		selection = sel;
	}
}
